# -*- encoding: utf-8 -*-


def is_positive_integer_power_of_two(expr):
    r'''Is true when `expr` is a positive integer power of ``2``.

    ::

        >>> for n in range(10):
        ...     print(n, mathtools.is_positive_integer_power_of_two(n))
        ... 
        0 False
        1 True
        2 True
        3 False
        4 True
        5 False
        6 False
        7 False
        8 True
        9 False

    Otherwise false.

    Returns boolean.
    '''
    from abjad.tools import mathtools

    return 0 < expr and mathtools.is_nonnegative_integer_power_of_two(expr)



# -*- encoding: utf-8 -*-


def transpose_pitch_class_number_to_pitch_number_neighbor(
    pitch_number, pitch_class_number):
    '''Transpose `pitch_class_number` by octaves to nearest neighbor
    of `pitch_number`:

    ::

        >>> pitchtools.transpose_pitch_class_number_to_pitch_number_neighbor(
        ...     12, 4)
        16

    Resulting pitch number must be within one tritone of `pitch_number`.

    Returns pitch number.
    '''

    target_pc = pitch_number % 12

    down = (target_pc - pitch_class_number) % 12
    up = (pitch_class_number - target_pc) % 12

    if up < down:
        return pitch_number + up
    else:
        return pitch_number - down


# -*- encoding: utf-8 -*-
from abjad import *



def test_scoretools_Container___delitem___01():
    r'''Delete spanned component.
    Component withdraws crossing spanners.
    Component carries covered spanners forward.
    Operation always leaves all expressions in tact.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, voice[:])
    slur = Slur()
    attach(slur, voice[0][:])
    slur = Slur()
    attach(slur, voice[1][:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8 )
            }
            {
                e'8 (
                f'8 ] )
            }
        }
        '''
        )

    old = voice[0]
    del(voice[0])

    "Container voice is now ..."

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                e'8 [ (
                f'8 ] )
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()

    "Deleted component is now ..."

    assert systemtools.TestManager.compare(
        old,
        r'''
        {
            c'8 (
            d'8 )
        }
        '''
        )

    assert inspect_(old).is_well_formed()


def test_scoretools_Container___delitem___02():
    r'''Delete 1 leaf in container.
    Spanner structure is preserved.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])

    del(voice[1])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            e'8
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___delitem___03():
    r'''Delete slice in middle of container.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])

    del(voice[1:3])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___delitem___04():
    r'''Delete slice from beginning to middle of container.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])

    del(voice[:2])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            e'8 [
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___delitem___05():
    r'''Delete slice from middle to end of container.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])

    del(voice[2:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___delitem___06():
    r'''Delete slice from beginning to end of container.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])

    del(voice[:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___delitem___07():
    r'''Delete leaf from tuplet.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    beam = Beam()
    attach(beam, tuplet[:])

    del(tuplet[1])

    assert systemtools.TestManager.compare(
        tuplet,
        r'''
        {
            c'8 [
            e'8 ]
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_Container___delitem___08():
    r'''Delete leaf from nested container.
    '''

    voice = Voice("c'8 { d'8 e'8 } f'8")
    beam = Beam()
    attach(beam, voice.select_leaves())
    glissando = spannertools.Glissando()
    attach(glissando, voice.select_leaves())

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [ \glissando
            {
                d'8 \glissando
                e'8 \glissando
            }
            f'8 ]
        }
        '''
        )

    leaf = voice.select_leaves()[1]
    del(voice[1][0])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [ \glissando
            {
                e'8 \glissando
            }
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert inspect_(leaf).is_well_formed()


# -*- encoding: utf-8 -*-
import math
import numbers
from abjad.tools import mathtools


def repeat_sequence_to_weight(sequence, weight, allow_total=Exact):
    '''Repeats `sequence` to `weight`.

    ..  container:: example

        Repeats sequence to weight of 23 exactly:

        ::

            >>> sequencetools.repeat_sequence_to_weight((5, -5, -5), 23)
            (5, -5, -5, 5, -3)

        Truncates last element when necessary.

    ..  container:: example

        Repeats sequence to weight of 23 more:

        ::

            >>> sequencetools.repeat_sequence_to_weight(
            ...     (5, -5, -5),
            ...     23,
            ...     allow_total=More,
            ...     )
            (5, -5, -5, 5, -5)

        Does not truncate last element.

    ..  container:: example

        Repeats sequence to weight of 23 or less:

        ::

            >>> sequencetools.repeat_sequence_to_weight(
            ...     (5, -5, -5),
            ...     23,
            ...     allow_total=Less,
            ...     )
            (5, -5, -5, 5)

        Discards last element when necessary.

    Returns newly constructed `sequence` object.
    '''

    # check input
    assert isinstance(weight, numbers.Number)
    assert 0 <= weight

    if allow_total == Exact:
        # repeat sequence and find overage
        sequence_weight = mathtools.weight(sequence)
        complete_repetitions = int(
            math.ceil(float(weight) / float(sequence_weight))
            )
        result = list(sequence)
        result = complete_repetitions * result
        overage = complete_repetitions * sequence_weight - weight
        # remove overage from result
        for element in reversed(result):
            if 0 < overage:
                element_weight = abs(element)
                candidate_overage = overage - element_weight
                if 0 <= candidate_overage:
                    overage = candidate_overage
                    result.pop()
                else:
                    absolute_amount_to_keep = element_weight - overage
                    assert 0 < absolute_amount_to_keep
                    signed_amount_to_keep = absolute_amount_to_keep
                    signed_amount_to_keep *= mathtools.sign(element)
                    result.pop()
                    result.append(signed_amount_to_keep)
                    break
            else:
                break
    elif allow_total == Less:
        # initialize result
        result = [sequence[0]]
        # iterate input
        i = 1
        while mathtools.weight(result) < weight:
            result.append(sequence[i % len(sequence)])
            i += 1
        # remove overage
        if weight < mathtools.weight(result):
            result = result[:-1]
        # return result
        return type(sequence)(result)
    elif allow_total == More:
        # initialize result
        result = [sequence[0]]
        # iterate input
        i = 1
        while mathtools.weight(result) < weight:
            result.append(sequence[i % len(sequence)])
            i += 1
        # return result
        return type(sequence)(result)
    else:
        message = 'is not an ordinal value constant: {!r}.'
        message = message.format(allow_total)
        raise ValueError(message)

    # return result
    return type(sequence)(result)


# -*- encoding: utf-8 -*-
import math
import numbers


def greatest_power_of_two_less_equal(n, i=0):
    r'''Greatest integer power of two less than or equal to positive `n`.

    ::

        >>> for n in range(10, 20):
        ...     print('\t%s\t%s' % (n, mathtools.greatest_power_of_two_less_equal(n)))
        ... 
            10 8
            11 8
            12 8
            13 8
            14 8
            15 8
            16 16
            17 16
            18 16
            19 16

    Greatest-but-``i`` integer power of ``2`` less than or
    equal to positive `n`:

    ::

        >>> for n in range(10, 20):
        ...     print('\t%s\t%s' % (n, mathtools.greatest_power_of_two_less_equal(n, i=1)))
        ... 
            10 4
            11 4
            12 4
            13 4
            14 4
            15 4
            16 8
            17 8
            18 8
            19 8

    Raises type error on nonnumeric `n`.

    Raises value error on nonpositive `n`.

    Returns positive integer.
    '''

    if not isinstance(n, numbers.Number):
        message = 'must be number: {!r}.'
        message = message.format(n)
        raise TypeError(message)

    if n <= 0:
        message = 'must be positive: {!r}.'
        message = message.format(n)
        raise ValueError(message)

    return 2 ** (int(math.log(n, 2)) - i)



def override(expr):
    r'''Overrides `expr`.

    Returns LilyPond grob manager.
    '''
    from abjad.tools import lilypondnametools

    if getattr(expr, '_lilypond_grob_name_manager', None) is None:
        manager = lilypondnametools.LilyPondGrobNameManager()
        expr._lilypond_grob_name_manager = manager

    return expr._lilypond_grob_name_manager


# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_label_leaves_in_expr_with_pitch_numbers_01():
    r'''Works on notes, rests and chords.
    '''

    leaves = scoretools.make_leaves([None, 12, (13, 14, 15), None], [(1, 4)])
    staff = Staff(leaves)
    labeltools.label_leaves_in_expr_with_pitch_numbers(staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            r4
            c''4
                _ \markup {
                    \small
                        12
                    }
            <cs'' d'' ef''>4
                _ \markup {
                    \column
                        {
                            \small
                                15
                            \small
                                14
                            \small
                                13
                        }
                    }
            r4
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


# -*- encoding: utf-8 -*-
import argparse
import os
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class DirectoryScript(DeveloperScript):
    '''`DirectoryScript` provides utilities for validating file system paths.

    `DirectoryScript` is abstract.
    '''

    ### PRIVATE METHODS ###

    def _is_valid_path(self, path):
        if os.path.exists(path):
            if os.path.isdir(path):
                return True
        return False

    def _validate_path(self, path):
        message = '{!r} is not a valid directory.'
        message = message.format(path)
        error = argparse.ArgumentTypeError(message)
        path = os.path.abspath(path)
        if not self._is_valid_path(path):
            raise error
        return os.path.relpath(path)


# -*- encoding: utf-8 -*-
import argparse
import os
import shutil
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript


class CleanScript(DirectoryScript):
    r'''Removes *.pyc, *.swp files and __pycache__ and tmp* directories
    recursively in a path.

    ..  shell::

        ajv clean --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'clean'``.
        '''
        return 'clean'

    @property
    def long_description(self):
        r'''Long description of scrip.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting groupt of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Clean *.pyc, *.swp, __pycache__ and tmp* files' + \
            ' and folders from PATH.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        if not args.pyc and not args.pycache and \
            not args.swp and not args.tmp:
            args.pyc, args.pycache, args.swp, args.tmp = True, True, True, True

        print('Cleaning...')
        if args.pyc:
            print('\t*.pyc files')
        if args.swp:
            print('\t*.swp files')
        if args.pycache:
            print('\t__pycache__ directories')
        if args.tmp:
            print('\ttmp* directories')

        for root_directory, directory_names, file_names in os.walk(args.path):
            if '.svn' in directory_names:
                directory_names.remove('.svn')
            extensions = ()
            if args.pyc:
                extensions += ('.pyc',)
            if args.swp:
                extensions += ('.swp',)
            for file_name in file_names:
                if file_name.endswith(extensions):
                    file_path = os.path.join(
                        root_directory,
                        file_name,
                        )
                    os.remove(file_path)
            directories_to_remove = []
            for directory_name in directory_names:
                directory = os.path.join(
                    root_directory,
                    directory_name,
                    )
                should_remove = False
                if args.pycache:
                    if directory_name == '__pycache__':
                        should_remove = True
                if args.tmp:
                    if directory_name.startswith('tmp'):
                        should_remove = True
                if not os.listdir(directory):
                    should_remove = True
                if should_remove:
                    shutil.rmtree(directory)
                    directories_to_remove.append(directory_name)
            for directory_name in directories_to_remove:
                directory_names.remove(directory_name)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument('path',
            default=os.getcwd(),
            help='directory tree to be recursed over',
            nargs='?',
            type=self._validate_path,
            )
        parser.add_argument('--pyc',
            action='store_true',
            help='delete *.pyc files',
            )
        parser.add_argument('--pycache',
            action='store_true',
            help='delete __pycache__ folders',
            )
        parser.add_argument('--swp',
            action='store_true',
            help='delete Vim *.swp file',
            )
        parser.add_argument('--tmp',
            action='store_true',
            help='delete tmp* folders',
            )


# -*- encoding: utf-8 -*-
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import select
from abjad.tools.scoretools.Context import Context


class Score(Context):
    r'''A score.

    ::

        >>> staff_1 = Staff("c'8 d'8 e'8 f'8")
        >>> staff_2 = Staff("c'8 d'8 e'8 f'8")
        >>> score = Score([staff_1, staff_2])
        >>> show(score) # doctest: +SKIP

    ..  doctest::

        >>> print(format(score))
        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        >>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, music=None, context_name='Score', name=None):
        Context.__init__(
            self,
            music=music,
            context_name=context_name,
            name=name,
            )
        self.is_simultaneous = True

    ### PUBLIC METHODS ###

    def add_final_bar_line(self, abbreviation='|.'):
        r'''Add final bar line to end of score.


            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> score = Score([staff])

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4
                    d'4
                    e'4
                    f'4
                }
            >>

        ::

            >>> show(score) # doctest: +SKIP

        ::

            >>> score.add_final_bar_line()
            BarLine('|.')

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4
                    d'4
                    e'4
                    f'4
                    \bar "|."
                }
            >>

        ::

            >>> show(score) # doctest: +SKIP

        Returns bar line.
        '''
        from abjad.tools import scoretools
        from abjad.tools import indicatortools
        selection = select(self)
        last_leaf = selection._get_component(scoretools.Leaf, -1)
        double_bar = indicatortools.BarLine(abbreviation)
        attach(double_bar, last_leaf)
        return double_bar

    def add_final_markup(self, markup, extra_offset=None):
        r'''Add `markup` to end of score:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> score = Score([staff])
            >>> markup = r'\italic \right-column { "Bremen - Boston - LA." "Jul 2010 - May 2011." }'
            >>> markup = markuptools.Markup(markup, Down)
            >>> markup = score.add_final_markup(markup, extra_offset=(4, -2))

        ::

            >>> print(format(markup, 'storage'))
            markuptools.Markup(
                contents=(
                    markuptools.MarkupCommand(
                        'italic',
                        markuptools.MarkupCommand(
                            'right-column',
                            ['Bremen - Boston - LA.', 'Jul 2010 - May 2011.']
                            )
                        ),
                    ),
                direction=Down,
                )

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    c'4
                    d'4
                    e'4
                    \once \override TextScript #'extra-offset = #'(4 . -2)
                    f'4 _ \markup {
                        \italic
                            \right-column
                                {
                                    "Bremen - Boston - LA."
                                    "Jul 2010 - May 2011."
                                }
                        }
                }
            >>

        ::

            >>> show(staff) # doctest: +SKIP

        Return `markup`.
        '''
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        selection = select(self)
        last_leaf = selection._get_component(scoretools.Leaf, -1)
        # TODO: copy markup direction from markup input
        markup = markup = markuptools.Markup(markup, Down)
        attach(markup, last_leaf)
        if extra_offset is not None:
            override(last_leaf).text_script.extra_offset = extra_offset
        return markup



# -*- encoding: utf-8 -*-
import collections
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.selectiontools.SimultaneousSelection \
    import SimultaneousSelection


class Parentage(SimultaneousSelection):
    r'''The parentage of a component.

    ..  container:: example

        ::

            >>> score = Score()
            >>> string = r"""\new Voice = "Treble Voice" { e'4 }"""
            >>> treble_staff = Staff(string, name='Treble Staff')
            >>> score.append(treble_staff)
            >>> string = r"""\new Voice = "Bass Voice" { c4 }"""
            >>> bass_staff = Staff(string, name='Bass Staff')
            >>> clef = Clef('bass')
            >>> attach(clef, bass_staff)
            >>> score.append(bass_staff)
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \context Staff = "Treble Staff" {
                    \context Voice = "Treble Voice" {
                        e'4
                    }
                }
                \context Staff = "Bass Staff" {
                    \clef "bass"
                    \context Voice = "Bass Voice" {
                        c4
                    }
                }
            >>

        ::

            >>> bass_voice = score['Bass Voice']
            >>> note = bass_voice[0]
            >>> parentage = inspect_(note).get_parentage()

        ::

            >>> for x in parentage: x
            ...
            Note('c4')
            Voice('c4')
            <Staff-"Bass Staff"{1}>
            <Score<<2>>>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_component',
        '_root',
        )

    ### INITIALIZER ###

    def __init__(self, component=None, include_self=True):
        from abjad.tools import scoretools
        assert isinstance(component, (scoretools.Component, type(None)))
        if component is None:
            music = ()
        else:
            music = []
            if include_self:
                parent = component
            else:
                parent = component._parent
            while parent is not None:
                music.append(parent)
                parent = parent._parent
            music = tuple(music)
        SimultaneousSelection.__init__(self, music)
        self._component = component

    ### PRIVATE PROPERTIES ###

    @property
    def _prolations(self):
        prolations = []
        default = durationtools.Multiplier(1)
        for parent in self:
            prolation = getattr(parent, 'implied_prolation', default)
            prolations.append(prolation)
        return prolations

    ### PRIVATE METHODS ###

    def _get_governor(self):
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        for component in self:
            if isinstance(component, scoretools.Container) and \
                not component.is_simultaneous:
                if component._parent is None:
                    return component
                if isinstance(component._parent, scoretools.Container) and \
                    component._parent.is_simultaneous:
                    return component

    @staticmethod
    def _id_string(component):
        lhs = component.__class__.__name__
        rhs = getattr(component, 'name', None) or id(component)
        return '{}-{!r}'.format(lhs, rhs)

    ### PUBLIC PROPERTIES ###

    @property
    def component(self):
        r'''The component from which the selection was derived.

        Returns component.
        '''
        return self._component

    @property
    def depth(self):
        r'''Length of proper parentage of component.

        Returns nonnegative integer.
        '''
        return len(self[1:])

    @property
    def is_orphan(self):
        r'''Is true when component has no parent.
        Otherwise false.

        Returns boolean.
        '''
        return self.parent is None

    @property
    def logical_voice(self):
        r'''Logical voice of component.

        ::

            >>> voice = Voice("c'4 d'4 e'4 f'4", name='CustomVoice')
            >>> staff = Staff([voice], name='CustomStaff')
            >>> score = Score([staff], name='CustomScore')
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \context Score = "CustomScore" <<
                \context Staff = "CustomStaff" {
                    \context Voice = "CustomVoice" {
                        c'4
                        d'4
                        e'4
                        f'4
                    }
                }
            >>

        ::

            >>> leaf = score.select_leaves()[0]
            >>> parentage = inspect_(leaf).get_parentage()
            >>> logical_voice = parentage.logical_voice

        ::

            >>> for key, value in logical_voice.items():
            ...     print('%12s: %s' % (key, value))
            ... 
                     score: Score-'CustomScore'
               staff group:
                     staff: Staff-'CustomStaff'
                     voice: Voice-'CustomVoice'

        Returns ordered dictionary.
        '''
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        keys = ('score', 'staff group', 'staff', 'voice')
        logical_voice = collections.OrderedDict.fromkeys(keys, '')
        for component in self:
            if isinstance(component, scoretools.Voice):
                if not logical_voice['voice']:
                    logical_voice['voice'] = self._id_string(component)
            elif isinstance(component, scoretools.Staff):
                if not logical_voice['staff']:
                    logical_voice['staff'] = self._id_string(component)
                    # explicit staff demands a nested voice:
                    # if no explicit voice has been found,
                    # create implicit voice here with random integer
                    if not logical_voice['voice']:
                        logical_voice['voice'] = id(component)
            elif isinstance(component, scoretools.StaffGroup):
                if not logical_voice['staff group']:
                    logical_voice['staff group'] = self._id_string(component)
            elif isinstance(component, scoretools.Score):
                if not logical_voice['score']:
                    logical_voice['score'] = self._id_string(component)
        return logical_voice

    @property
    def parent(self):
        r'''Parent of component.

        Returns none when component has no parent.

        Returns component or none.
        '''
        if 1 < len(self):
            return self[1]

    @property
    def prolation(self):
        r'''Prolation governing component.

        Returns multiplier.
        '''
        prolations = [durationtools.Multiplier(1)] + self._prolations
        products = mathtools.cumulative_products(prolations)
        return products[-1]

    @property
    def root(self):
        r'''Last element in parentage.

        Returns component.
        '''
        return self[-1]

    @property
    def score_index(self):
        r'''Score index of component.

        ::

            >>> staff_1 = Staff(r"\times 2/3 { c''2 b'2 a'2 }")
            >>> staff_2 = Staff("c'2 d'2")
            >>> score = Score([staff_1, staff_2])
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    \times 2/3 {
                        c''2
                        b'2
                        a'2
                    }
                }
                \new Staff {
                        c'2
                        d'2
                }
            >>

        ::

            >>> leaves = score.select_leaves(allow_discontiguous_leaves=True)
            >>> for leaf in leaves:
            ...     parentage = inspect_(leaf).get_parentage()
            ...     leaf, parentage.score_index
            ...
            (Note("c''2"), (0, 0, 0))
            (Note("b'2"), (0, 0, 1))
            (Note("a'2"), (0, 0, 2))
            (Note("c'2"), (1, 0))
            (Note("d'2"), (1, 1))

        Returns tuple of zero or more nonnegative integers.
        '''
        result = []
        current = self[0]
        for parent in self[1:]:
            index = parent.index(current)
            result.insert(0, index)
            current = parent
        result = tuple(result)
        return result

    @property
    def tuplet_depth(self):
        r'''Tuplet depth of component.

        ::

            >>> tuplet = Tuplet(Multiplier(2, 3), "c'2 d'2 e'2")
            >>> staff = Staff([tuplet])
            >>> note = staff.select_leaves()[0]
            >>> show(staff) # doctest: +SKIP

        ::

            >>> inspect_(note).get_parentage().tuplet_depth
            1

        ::

            >>> inspect_(tuplet).get_parentage().tuplet_depth
            0

        ::

            >>> inspect_(staff).get_parentage().tuplet_depth
            0

        Returns nonnegative integer.
        '''
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        result = 0
        # should probably interate up to only first simultaneous container
        # in parentage.
        # note that we probably need a named idea for 'parentage
        # up to first simultaneous container'.
        for parent in self[1:]:
            if isinstance(parent, scoretools.Tuplet):
                result += 1
        return result

    ### PUBLIC METHODS ###

    def get_first(self, prototype=None):
        r'''Gets first instance of `prototype` in parentage.

        Returns component or none.
        '''
        from abjad.tools import scoretools
        if prototype is None:
            prototype = (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        for component in self:
            if isinstance(component, prototype):
                return component



# -*- encoding: utf-8 -*-
import abc
import collections
from abjad.tools.datastructuretools.TypedCollection import TypedCollection
from abjad.tools.topleveltools import new


class TypedOrderedDict(TypedCollection):
    r'''A typed ordered dictionary.

    ..  note:: Doctests not included because class is used as a
        base class. Doctests here would put weird examples in child classes.
        See child classes for doctests.

    '''

    ### CLASS VARIABLES ###

    __metaclass__ = abc.ABCMeta

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        TypedCollection.__init__(
            self,
            item_class=item_class,
            items=items,
            )
        if isinstance(items, collections.Mapping):
            items = items.items()
        items = items or []
        the_items = []
        for item in items:
            assert len(item) == 2, repr(item)
            key = item[0]
            value = self._item_callable(item[1])
            the_item = (key, value)
            the_items.append(the_item)
        self._collection = collections.OrderedDict(items)

    ### SPECIAL METHODS ###

    def __cmp__(self, expr):
        r'''Aliases OrderedDict.__cmp__().

        Returns boolean.
        '''
        assert isinstance(expr, type(self))
        ordered_dictionary = expr._collection
        return self._collection.__cmp__(ordered_dictionary)

    def __contains__(self, key):
        r'''Aliases OrderedDict.__contains__().

        Returns boolean.
        '''
        return key in self._collection

    def __delitem__(self, i):
        r'''Aliases OrderedDict.__delitem__().

        Returns none.
        '''
        del(self._collection[i])

    def __ge__(self, expr):
        r'''Is true when typed ordered dictionary is greater than or equal
        to `expr`. Otherwise false.

        Returns boolean.
        '''
        expr = type(self)(expr)
        return self._collection.__ge__(expr._collection)

    def __getitem__(self, i):
        r'''Aliases OrderedDict.__getitem__().

        Returns item.
        '''
        return self._collection[i]

    def __gt__(self, expr):
        r'''Is true when typed ordered dictionary is greater than `expr`.
        Otherwise false.

        Returns boolean.
        '''
        expr = type(self)(expr)
        return self._collection.__gt__(expr._collection)

    def __le__(self, expr):
        r'''Is true when typed ordered dictionary is less than or equal
        to `expr`. Otherwise false.

        Returns boolean.
        '''
        expr = type(self)(expr)
        return self._collection.__le__(expr._collection)

    def __lt__(self, expr):
        r'''Is true when typed ordered dictionary is less than `expr`.
        Otherwise false.

        Returns boolean.
        '''
        expr = type(self)(expr)
        return self._collection.__lt__(expr._collection)

    def __ne__(self, expr):
        r'''Is true when typed ordered dictionary is not equal to `expr`.
        Otherwise false.

        Returns boolean.
        '''
        return not self == expr

    def __reversed__(self):
        r'''Aliases OrderedDict.__reversed__().

        Returns generatos.
        '''
        return self._collection.__reversed__()

    def __setitem__(self, i, expr):
        r'''Changes items in `expr` to items and sets.

        Returns none.
        '''
        new_item = self._item_callable(expr)
        self._collection[i] = new_item

    ### PUBLIC METHODS ###

    def clear(self):
        r'''Clears typed ordered dictionary.

        Returns none.
        '''
        self._collection.clear()

    def copy(self):
        r'''Copies typed ordered dictionary.

        Returns new typed ordered dictionary.
        '''
        ordered_dictionary = self._collection.copy()
        items = list(ordered_dictionary.items())
        return type(self)(
            item_class=self.item_class,
            items=items,
            )

    def get(self, i, default=None):
        r'''Aliases OrderedDict.get().

        Returns item or raises key error.
        '''
        return self._collection.get(i, default)

    def has_key(self, key):
        r'''Aliases OrderdDict.has_key().

        Returns boolean.
        '''
        return key in self._collection

    def items(self):
        r'''Aliases OrderedDict.items().

        Returns list.
        '''
        return list(self._collection.items())

    def items(self):
        r'''Aliases OrderedDict.items().

        Returns generator.
        '''
        return iter(self._collection.items())

    def keys(self):
        r'''Aliases OrderedDict.keys().

        Returns generator.
        '''
        return iter(self._collection.keys())

    def pop(self, key, default=None):
        r'''Aliases OrderedDict.pop().

        Returns items.
        '''
        return self._collection.pop(key, default)

    def popitem(self):
        r'''Aliases OrderedDict.popitem().

        Returns generator.
        '''
        return self._collection.popitem()

    def setdefault(self, key, default=None):
        r'''Aliases OrderedDict.setdefault().

        Returns items.
        '''
        return self._collection.setdefault(key, default)

    def update(self, *args, **kwargs):
        r'''Aliases OrderedDict.update().

        Returns none.
        '''
        return self._collection.update(*args, **kwargs)

    def values(self):
        r'''Aliases OrderedDict.values().

        Returns generator.
        '''
        return iter(self._collection.values())


collections.MutableMapping.register(TypedOrderedDict)



# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_leaves_in_expr_with_pitch_numbers(expr, markup_direction=Down):
    r'''Label leaves in `expr` with pitch numbers:

    ::

        >>> staff = Staff(scoretools.make_leaves([None, 12, [13, 14, 15], None], [(1, 4)]))
        >>> labeltools.label_leaves_in_expr_with_pitch_numbers(staff)
        >>> print(format(staff))
        \new Staff {
            r4
            c''4 _ \markup { \small 12 }
            <cs'' d'' ef''>4 _ \markup { \column { \small 15 \small 14 \small 13 } }
            r4
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    '''

    for leaf in iterate(expr).by_class(scoretools.Leaf):
        for pitch in reversed(pitchtools.PitchSegment.from_selection(leaf)):
            if pitch is not None:
                label = markuptools.MarkupCommand('small', str(pitch.pitch_number))
                markup = markuptools.Markup(label, markup_direction)
                attach(markup, leaf)



# -*- encoding: utf-8 -*-


def select(expr):
    r'''Select `expr` for tonal analysis.

    Returns tonal analysis selection.
    '''
    from abjad.tools import scoretools
    from abjad.tools import tonalanalysistools
    if isinstance(expr, scoretools.Component):
        return tonalanalysistools.TonalAnalysisAgent(expr)
    elif hasattr(expr, '_music'):
        music = expr._music
        return tonalanalysistools.TonalAnalysisAgent(music)
    else:
        return tonalanalysistools.TonalAnalysisAgent(expr)


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_sequencetools_rotate_sequence_01():
    r'''Rotates sequence by distance less than or equal to sequence length.
    '''

    assert sequencetools.rotate_sequence(list(range(10)), -3) == [3, 4, 5, 6, 7, 8, 9, 0, 1, 2]
    assert sequencetools.rotate_sequence(list(range(10)), 4) == [6, 7, 8, 9, 0, 1, 2, 3, 4, 5]
    assert sequencetools.rotate_sequence(list(range(10)), 0) == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]


def test_sequencetools_rotate_sequence_02():
    r'''Rotates sequence by distance greatern than sequence length.
    '''

    assert sequencetools.rotate_sequence(list(range(10)), -23) == [3, 4, 5, 6, 7, 8, 9, 0, 1, 2]
    assert sequencetools.rotate_sequence(list(range(10)), 24) == [6, 7, 8, 9, 0, 1, 2, 3, 4, 5]


def test_sequencetools_rotate_sequence_03():
    r'''Returns sequence type.
    '''

    sequence = list(range(10))
    new = sequencetools.rotate_sequence(sequence, -1)
    assert isinstance(new, type(sequence))

    sequence = tuple(range(10))
    new = sequencetools.rotate_sequence(sequence, -1)
    assert isinstance(new, type(sequence))


def test_sequencetools_rotate_sequence_04():
    r'''Rotates named pitch segment.
    '''
    pytest.skip('FIXME')

    named_pitch_segment_1 = pitchtools.PitchSegment("c'' d'' e'' f''")
    named_pitch_segment_2 = sequencetools.rotate_sequence(named_pitch_segment_1, -1)
    named_pitch_segment_3 = pitchtools.PitchSegment("d'' e'' f'' c''")

    assert named_pitch_segment_2 == named_pitch_segment_3
    assert isinstance(named_pitch_segment_1, pitchtools.PitchSegment)
    assert isinstance(named_pitch_segment_2, pitchtools.PitchSegment)
    assert isinstance(named_pitch_segment_3, pitchtools.PitchSegment)



# -*- encoding: utf-8 -*-
import argparse
import os
import sys
import traceback
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class AbjadBookScript(DeveloperScript):
    r'''abjad book script.
    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of Abjad book script.

        Returns ``'book'``.
        '''
        return 'book'

    @property
    def long_description(self):
        r'''Long description of Abjad book script.

        Returns string.
        '''
        return r'''DESCRIPTION

    abjad-book processes Abjad snippets embedded in HTML, LaTeX, or ReST
    documents. All Abjad code placed between the <abjad> </abjad> tags in
    either HTML, LaTeX or ReST type documents is executed and replaced with
    tags appropriate to the given file type. All output generated by the
    code snippet is captured and inserted in the output file.

    Apart from the special opening and closing Abjad tags, abjad-book also
    has a special line-level suffix tag: `<hide`. All lines ending with the
    `<hide` tag will be interpreted by Abjad but will not be displayed in the
    OUTPUT document.

    The opening <abjad> tag can also be followed by a list of
    `attribute=value` pair.

    You can make all of an Abjad code block invisible in the output file with
    the following opening tag:

    <abjad>[hide=true]

    This is useful for generating and embedding rendered score images without
    showing any of the Abjad code.

    You can also remove all of the prompts from a code block with the
    following opening tag:

    <abjad>[strip_prompt=true]

    Simply use Abjad's show() function to have Abjad call LilyPond on the
    Abjad snippet and embed the rendered image in the document.

    All Abjad snippets *must* start with no indentation in the document.

EXAMPLES

    1.  Create an HTML, LaTex or ReST document with embedded Abjad code
        between <abjad></abjad> tags. The code *must* be fully flushed
        to the left, with no tabs or spaces. The content of an HTML file
        with embedded Abjad might look like this:

        This is an <b>HTML</b> document. Here is Abjad code:

        <abjad>
        voice = Voice("c'4 d'4 e'4 f'4")
        beam = spannertools.Beam()
        attach(beam, voice)
        show(voice)
        </abjad>

        More ordinary <b>HTML</b> text here.

   2.  Call `abjad-book` on the file just created:

        $ abjad-book file.htm.raw
    '''

    ### PRIVATE METHODS ###

    def _collect_file_names(self, args):
        file_names = []
        if os.path.isdir(args.path):
            for root, dirs, files in os.walk(args.path):
                for file in files:
                    path = os.path.join(root, file)
                    if self._is_valid_path(path):
                        file_names.append(path)
        else:
            file_names.append(args.path)
        return file_names

    def _is_valid_path(self, path):
        if not os.path.exists(path):
            return False
        if os.path.isfile(path) and \
            path.endswith('.raw') and \
            path.rpartition('.raw')[0].rpartition('.')[-1] in \
            self.output_formats:
            return True
        if os.path.isdir(path):
            return True
        return False

    def _process_file_name(self, args, file_name):
        from abjad.tools import abjadbooktools
        input_file_name = file_name
        print('Processing {!r} ...'.format(os.path.relpath(input_file_name)))
        try:
            directory = os.path.dirname(input_file_name)
            output_file_name = input_file_name.rpartition('.raw')[0]
            file_extension = output_file_name.rpartition('.')[-1]
            image_prefix = os.path.basename(output_file_name).rpartition('.')[0]
            with open(input_file_name, 'r') as f:
                lines = f.read().split('\n')
            output_format = self.output_formats[file_extension]()
            abjad_book_processor = abjadbooktools.AbjadBookProcessor(
                directory, lines, output_format,
                skip_rendering=args.skip_rendering, image_prefix=image_prefix,
                verbose=args.verbose)
            processed_lines = abjad_book_processor(verbose=args.verbose)
            print('Writing output to {!r} ...'.format(
                os.path.relpath(output_file_name)))
            with open(output_file_name, 'w') as f:
                f.write(processed_lines)
            print('... Done!')
        except:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            traceback.print_exc()

    def _validate_path(self, path):
        message = 'not a valid path: {!r}.'
        message = message.format(path)
        error = argparse.ArgumentTypeError(message)
        path = os.path.abspath(path)
        if not self._is_valid_path(path):
            raise error
        return path

    ### PUBLIC PROPERTIES ###

    @property
    def output_formats(self):
        r'''Output formats of Abjad book script.

        Returns dictionary.
        '''
        from abjad.tools import abjadbooktools
        return {
            'htm': abjadbooktools.HTMLOutputFormat,
            'html': abjadbooktools.HTMLOutputFormat,
            'rst': abjadbooktools.ReSTOutputFormat,
            'tex': abjadbooktools.LaTeXOutputFormat,
        }

    @property
    def short_description(self):
        r'''Short description of Abjad book script.

        Returns string.
        '''
        return 'Preprocess HTML, LaTeX or ReST source with Abjad.'

    @property
    def version(self):
        r'''Version of Abjad book script.

        Returns float.
        '''
        return 2.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        from abjad.tools import developerscripttools
        for file_name in self._collect_file_names(args):
            self._process_file_name(args, file_name)
        flags = []
        if args.mainline:
            flags.append('-M')
        if args.experimental:
            flags.append('-X')
        if flags:
            developerscripttools.BuildApiScript()(flags)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument('path',
            default=os.getcwd(),
            help='directory tree to be recursed over',
            nargs='?',
            type=self._validate_path,
            )
        parser.add_argument('--skip-rendering',
            action='store_true',
            help='skip all image rendering and simply execute the code',
            )
        parser.add_argument('--verbose',
            action='store_true',
            dest='verbose',
            help='run in verbose mode, printing all LilyPond output',
            )
        parser.add_argument('-X', '--experimental',
            action='store_true',
            help='rebuild abjad.tools docs after processing',
            )
        parser.add_argument('-M', '--mainline',
            action='store_true',
            help='rebuild mainline docs after processing',
            )


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_Tempo___add___01():

    tempo_indication_1 = Tempo(Duration(1, 4), 60)
    tempo_indication_2 = Tempo(Duration(1, 4), 90)

    result = tempo_indication_1 + tempo_indication_2
    assert result == Tempo(Duration(1, 4), 150)

    result = tempo_indication_2 + tempo_indication_1
    assert result == Tempo(Duration(1, 4), 150)


def test_indicatortools_Tempo___add___02():

    tempo_indication_1 = Tempo(Duration(1, 8), 42)
    tempo_indication_2 = Tempo(Duration(1, 4), 90)

    result = tempo_indication_1 + tempo_indication_2
    assert result == Tempo(Duration(1, 4), 174)

    result = tempo_indication_2 + tempo_indication_1
    assert result == Tempo(Duration(1, 4), 174)


def test_indicatortools_Tempo___add___03():

    tempo_indication_1 = Tempo(textual_indication='Langsam')
    tempo_indication_2 = Tempo(Duration(1, 4), 90)

    statement = "tempo_indication_1 + tempo_indication_2"
    pytest.raises(ImpreciseTempoError, statement)


def test_indicatortools_Tempo___add___04():

    tempo_indication_1 = Tempo(Duration(1, 8), (90, 92))
    tempo_indication_2 = Tempo(Duration(1, 4), 90)

    statement = "tempo_indication_1 + tempo_indication_2"
    pytest.raises(ImpreciseTempoError, statement)


# -*- encoding: utf-8 -*-
import pickle
from abjad import *


def test_documentationtools_GraphvizGraph_pickle_01():

    graph = documentationtools.GraphvizGraph()
    graph.append(documentationtools.GraphvizSubgraph())
    graph[0].append(documentationtools.GraphvizNode())
    graph[0].append(documentationtools.GraphvizNode())
    graph[0].append(documentationtools.GraphvizNode())
    graph[0].append(documentationtools.GraphvizSubgraph())
    graph[0][-1].append(documentationtools.GraphvizNode())
    graph.append(documentationtools.GraphvizNode())
    edge = documentationtools.GraphvizEdge()(graph[0][1], graph[1])
    edge = documentationtools.GraphvizEdge()(graph[0][0], graph[0][-1][0])

    assert systemtools.TestManager.compare(
        graph.graphviz_format,
        r'''
        digraph Graph {
            subgraph cluster_0 {
                node_0_0;
                node_0_1;
                node_0_2;
                subgraph cluster_0_3 {
                    node_0_3_0;
                }
                node_0_0 -> node_0_3_0;
            }
            node_1;
            node_0_1 -> node_1;
        }
        '''
        )

    pickled = pickle.loads(pickle.dumps(graph))

    assert graph.graphviz_format == pickled.graphviz_format


# -*- encoding: utf-8 -*-
import inspect
import pickle
import pytest
import abjad
from abjad.tools import datastructuretools
from abjad.tools import documentationtools
from abjad.tools import lilypondparsertools
from abjad.tools import quantizationtools
from abjad.tools import rhythmtreetools
from abjad.tools import selectiontools
from abjad.tools import systemtools
from abjad.tools import tonalanalysistools


_classes_to_fix = (
    documentationtools.ClassDocumenter,
    documentationtools.InheritanceGraph,
    documentationtools.Pipe,
    lilypondparsertools.LilyPondParser,
    lilypondparsertools.SchemeParser,
    lilypondparsertools.ReducedLyParser,
    quantizationtools.ParallelJobHandlerWorker,
    rhythmtreetools.RhythmTreeParser,
    selectiontools.ContiguousSelection,
    selectiontools.Descendants,
    selectiontools.LogicalTie,
    selectiontools.Lineage,
    selectiontools.Parentage,
    selectiontools.Selection,
    selectiontools.SimultaneousSelection,
    selectiontools.SliceSelection,
    selectiontools.VerticalMoment,
    systemtools.LilyPondFormatBundle,
    systemtools.LilyPondFormatBundle.SlotContributions,
    systemtools.RedirectedStreams,
    tonalanalysistools.TonalAnalysisAgent,
    )

classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_abjad_pickle_01(class_):
    r'''All storage-formattable classes are pickable.
    '''

    if '_storage_format_specification' in dir(class_):
        if not inspect.isabstract(class_):
            if class_ not in _classes_to_fix:
                instance_one = class_()
                pickle_string = pickle.dumps(instance_one)
                instance_two = pickle.loads(pickle_string)
                instance_one_format = format(instance_one, 'storage')
                instance_two_format = format(instance_two, 'storage')
                assert instance_one_format == instance_two_format


# -*- encoding: utf-8 -*-
from abjad.tools.scoretools.Container import Container


class GraceContainer(Container):
    r'''A container of grace music.

    ::

        >>> voice = Voice("c'8 d'8 e'8 f'8")
        >>> beam = spannertools.Beam()
        >>> attach(beam, voice[:])
        >>> show(voice) # doctest: +SKIP

    ..  doctest::

        >>> print(format(voice))
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> grace_notes = [Note("c'16"), Note("d'16")]
        >>> grace_container = scoretools.GraceContainer(grace_notes, kind='grace')
        >>> attach(grace_container, voice[1])
        >>> show(voice) # doctest: +SKIP

    ..  doctest::

        >>> print(format(voice))
        \new Voice {
            c'8 [
            \grace {
                c'16
                d'16
            }
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> notes = [Note("e'16"), Note("f'16")]
        >>> after_grace = scoretools.GraceContainer(notes, kind='after')
        >>> attach(after_grace, voice[1])
        >>> show(voice) # doctest: +SKIP

    ..  doctest::

        >>> print(format(voice))
        \new Voice {
            c'8 [
            \grace {
                c'16
                d'16
            }
            \afterGrace
            d'8
            {
                e'16
                f'16
            }
            e'8
            f'8 ]
        }

    Fill grace containers with notes, rests or chords.

    Attach grace containers to nongrace notes, rests or chords.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_carrier',
        '_kind',
        )

    ### INITIALIZER ###

    def __init__(self, music=None, kind='grace'):
        # self._carrier is a reference to the leaf carrying grace music
        self._carrier = None
        Container.__init__(self, music)
        self.kind = kind

#    ### SPECIAL METHODS ###
#
#    def __repr__(self):
#        r'''Gets interpreter representation of grace container.
#
#        Returns string.
#        '''
#        return '{}({})'.format(type(self).__name__, self._summary)

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        self._update_now(indicators=True)
        return self._format_component()

    ### PRIVATE METHODS ###

    def _attach(self, arg):
        from abjad.tools import scoretools
        if not isinstance(arg, scoretools.Leaf):
            message = 'object to which grace container attaches'
            message += ' must be leaf: {!r}.'
            message = message.format(arg)
            raise TypeError(message)
        if self.kind == 'after':
            arg._after_grace = self
        else:
            arg._grace = self
        self._carrier = arg

    def _copy_with_indicators_but_without_children_or_spanners(self):
        new = Container._copy_with_indicators_but_without_children_or_spanners(self)
        new.kind = self.kind
        return new

    def _detach(self):
        if self._carrier is not None:
            carrier = self._carrier
            if self.kind == 'after':
                delattr(carrier, '_after_grace')
            else:
                delattr(carrier, '_grace')
            self._carrier = None
            self[:] = []
        return self

    def _format_open_brackets_slot(self, bundle):
        result = []
        kind = self.kind
        if kind == 'after':
            result.append([('grace_brackets', 'open'), ['{']])
        else:
            contributor = ('grace_brackets', 'open')
            contributions = [r'\{} {{'.format(kind)]
            result.append([contributor, contributions])
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def kind(self):
        r'''Gets `kind` of grace container.

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> note = Note("cs'16")
            >>> grace = scoretools.GraceContainer([note], kind='grace')
            >>> attach(grace, staff[1])
            >>> grace.kind
            'grace'

        Sets `kind` of grace container:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> note = Note("cs'16")
            >>> grace = scoretools.GraceContainer([note], kind='grace')
            >>> attach(grace, staff[1])
            >>> grace.kind = 'acciaccatura'
            >>> grace.kind
            'acciaccatura'

        Valid options include ``'after'``, ``'grace'``,
        ``'acciaccatura'``, ``'appoggiatura'``.

        Returns string.
        '''
        return self._kind

    @kind.setter
    def kind(self, arg):
        assert arg in (
            'after',
            'grace',
            'acciaccatura',
            'appoggiatura',
            )
        self._kind = arg



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchArray__get_composite_offset_difference_series_from_leavs_in_expr_01():

    staff_1 = Staff(r"\times 4/3 { c'8 d'8 e'8 }")
    staff_2 = Staff("f'8 g'8 a'8 b'8")
    score = Score([staff_1, staff_2])

    assert systemtools.TestManager.compare(
        score,
        r'''
        \new Score <<
            \new Staff {
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 4/3 {
                    c'8
                    d'8
                    e'8
                }
            }
            \new Staff {
                f'8
                g'8
                a'8
                b'8
            }
        >>
        '''
        )

    result = pitchtools.PitchArray._get_composite_offset_difference_series_from_leaves_in_expr(score)

    assert result == [
        Duration(1, 8),
        Duration(1, 24),
        Duration(1, 12),
        Duration(1, 12),
        Duration(1, 24),
        Duration(1, 8),
        ]


# -*- encoding: utf-8 -*-
from abjad import *


def test_Sequence_is_restricted_growth_function_01():

    assert Sequence(1, 1, 1, 1).is_restricted_growth_function()
    assert Sequence(1, 1, 1, 2).is_restricted_growth_function()
    assert Sequence(1, 1, 2, 1).is_restricted_growth_function()
    assert Sequence(1, 1, 2, 2).is_restricted_growth_function()
    assert Sequence(1, 1, 2, 3).is_restricted_growth_function()
    assert Sequence(1, 2, 1, 1).is_restricted_growth_function()
    assert Sequence(1, 2, 1, 2).is_restricted_growth_function()
    assert Sequence(1, 2, 1, 3).is_restricted_growth_function()
    assert Sequence(1, 2, 2, 1).is_restricted_growth_function()
    assert Sequence(1, 2, 2, 2).is_restricted_growth_function()
    assert Sequence(1, 2, 2, 3).is_restricted_growth_function()
    assert Sequence(1, 2, 3, 1).is_restricted_growth_function()
    assert Sequence(1, 2, 3, 2).is_restricted_growth_function()
    assert Sequence(1, 2, 3, 3).is_restricted_growth_function()
    assert Sequence(1, 2, 3, 4).is_restricted_growth_function()


def test_Sequence_is_restricted_growth_function_02():

    assert not Sequence(1, 1, 1, 3).is_restricted_growth_function()
    assert not Sequence(1, 1, 3, 3).is_restricted_growth_function()
    assert not Sequence(1, 3, 1, 3).is_restricted_growth_function()
    assert not Sequence(3, 1, 1, 3).is_restricted_growth_function()


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


# TODO: extend to attach to spanners
class LilyPondCommand(AbjadObject):
    r'''A LilyPond command.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> slur = spannertools.Slur()
        >>> attach(slur, staff.select_leaves())

    ::

        >>> command = indicatortools.LilyPondCommand('slurDotted')
        >>> attach(command, staff[0])

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            \slurDotted
            c'8 (
            d'8
            e'8
            f'8 )
        }

    ::

        >>> show(staff) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_format_slot',
        '_name',
        )

    _format_leaf_children = False

    _valid_format_slots = (
        'before',
        'after',
        'opening',
        'closing',
        'right',
        )

    ### INITIALIZER ###

    def __init__(self, name=None, format_slot=None):
        name = name or 'slurDotted'
        format_slot = format_slot or 'opening'
        assert format_slot in self._valid_format_slots, repr(format_slot)
        assert isinstance(name, str), repr(name)
        self._name = name
        self._format_slot = format_slot

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies LilyPond command.

        Returns new LilyPond command.
        '''
        new = type(self)(self._name)
        new._format_slot = self.format_slot
        return new

    def __eq__(self, expr):
        r'''Is true when `expr` is a LilyPond command with a name equal to
        that of this LilyPond command. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self._name == expr._name
        return False

    def __format__(self, format_specification=''):
        r'''Formats LilyPond command.

        Set `format_specification` to `''`, `'lilypond`' or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        elif format_specification == 'lilypond':
            return self._lilypond_format
        return str(self)

    def __hash__(self):
        r'''Hashes LilyPond command.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(LilyPondCommand, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self.name)

    @property
    def _lilypond_format(self):
        from abjad.tools import stringtools
        command = self._name
        if command.startswith('#'):
            return command
        else:
            return '\\' + stringtools.snake_case_to_lower_camel_case(command)

    @property
    def _lilypond_format_bundle(self):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        format_slot = lilypond_format_bundle.get(self.format_slot)
        format_slot.commands.append(self._lilypond_format)
        return lilypond_format_bundle

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = (
            self.name,
            )
        keyword_argument_names = ()
        if not self.format_slot == 'opening':
            keyword_argument_names = (
                'format_slot',
                )
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=keyword_argument_names,
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def format_slot(self):
        r'''Gets format slot of LilyPond command.

        ::

            >>> command.format_slot
            'opening'

        Returns string.
        '''
        return self._format_slot


    @property
    def name(self):
        r'''Gets name of LilyPond command.

        ::

            >>> command.name
            'slurDotted'

        Returns string.
        '''
        return self._name


# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_Matrix_rows_01():

    matrix = datastructuretools.Matrix([
        [0, 1, 2, 3],
        [10, 11, 12, 13],
        [20, 21, 22, 23],
        ])

    assert matrix.rows == ((0, 1, 2, 3), (10, 11, 12, 13), (20, 21, 22, 23))
    assert matrix.rows[2] == (20, 21, 22, 23)
    assert matrix.rows[2][0] == 20


# -*- encoding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner


class Tie(Spanner):
    r'''A collection of consecutive ties.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 c'8 c'8 c'8")
            >>> tie = Tie()
            >>> attach(tie, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'8 ~
                c'8 ~
                c'8 ~
                c'8
            }

    Formats LilyPond ``~`` command on nonlast leaves in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_direction',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        direction=None,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        direction = stringtools.arg_to_tridirectional_lilypond_symbol(
            direction)
        self._direction = direction

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._direction = self.direction

    def _format_right_of_leaf(self, leaf):
        from abjad.tools import scoretools
        result = []
        prototype = (
            scoretools.Container,
            scoretools.Rest,
            scoretools.Skip,
            scoretools.MultimeasureRest,
            )
        if self._is_my_last_leaf(leaf):
            return result
        elif isinstance(leaf, prototype):
            return result
        elif isinstance(leaf._get_leaf(1), prototype):
            return result
        if self.direction is not None:
            string = '{} ~'.format(self.direction)
            result.append(string)
        else:
            result.append('~')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Gets direction of ties in spanner.

        ..  container:: example

            Forces ties up:

            ::

                >>> staff = Staff("c'8 c'8 c'8 c'8")
                >>> tie = Tie(direction=Up)
                >>> attach(tie, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff))
                \new Staff {
                    c'8 ^ ~
                    c'8 ^ ~
                    c'8 ^ ~
                    c'8
                }

            ::

                >>> tie.direction
                '^'

        ..  container:: example

            Forces ties down:

            ::

                >>> staff = Staff("c'8 c'8 c'8 c'8")
                >>> tie = Tie(direction=Down)
                >>> attach(tie, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff))
                \new Staff {
                    c'8 _ ~
                    c'8 _ ~
                    c'8 _ ~
                    c'8
                }

            ::

                >>> tie.direction
                '_'

        ..  container:: example

            Positions ties according to LilyPond defaults:

            ::

                >>> staff = Staff("c'8 c'8 c'8 c'8")
                >>> tie = Tie(direction=None)
                >>> attach(tie, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print(format(staff))
                \new Staff {
                    c'8 ~
                    c'8 ~
                    c'8 ~
                    c'8
                }

            ::

                >>> tie.direction is None
                True

        Returns up, down or none.
        '''
        return self._direction



# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Instrument import Instrument


class ClarinetInBFlat(Instrument):
    r'''A B-flat clarinet.

    ::

        >>> staff = Staff("c'4 d'4 e'4 fs'4")
        >>> clarinet = instrumenttools.ClarinetInBFlat()
        >>> attach(clarinet, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            \set Staff.instrumentName = \markup { Clarinet in B-flat }
            \set Staff.shortInstrumentName = \markup { Cl. in B-flat }
            c'4
            d'4
            e'4
            fs'4
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    performer_abbreviation = 'cl.'

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='clarinet in B-flat',
        short_instrument_name='cl. in B-flat',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=None,
        pitch_range='[D3, Bb6]',
        sounding_pitch_of_written_middle_c='Bb3',
        ):
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._performer_names.extend([
            'wind player',
            'reed player',
            'single reed player',
            'clarinettist',
            'clarinetist',
            ])
        self._is_primary_instrument = True

    ### PRIVATE METHODS ###

    def _get_performer_names(self):
        r'''Get performer names:

        ::

            >>> for performer_name in clarinet._get_performer_names():
            ...     performer_name
            'instrumentalist'
            'wind player'
            'reed player'
            'single reed player'
            'clarinettist'
            'clarinetist'

        Returns list.
        '''
        return super(ClarinetInBFlat, self)._get_performer_names()

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets clarinet in B-flat's allowable clefs.

        ..  container:: example

            ::

                >>> clarinet.allowable_clefs
                ClefInventory([Clef(name='treble')])

            ::

                >>> show(clarinet.allowable_clefs) # doctest: +SKIP

        Returns clef inventory.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def instrument_name(self):
        r'''Gets clarinet in B-flat's name.

        ..  container:: example

            ::

                >>> clarinet.instrument_name
                'clarinet in B-flat'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets clarinet in B-flat's instrument name markup.

        ..  container:: example

            ::

                >>> clarinet.instrument_name_markup
                Markup(contents=('Clarinet in B-flat',))

            ::

                >>> show(clarinet.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets clarinet in B-flat's range.

        ..  container:: example

            ::

                >>> clarinet.pitch_range
                PitchRange(range_string='[D3, Bb6]')

            ::

                >>> show(clarinet.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets clarinet in B-flat's short instrument name.

        ..  container:: example

            ::

                >>> clarinet.short_instrument_name
                'cl. in B-flat'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets clarinet in B-flat's short instrument name markup.

        ..  container:: example

            ::

                >>> clarinet.short_instrument_name_markup
                Markup(contents=('Cl. in B-flat',))

            ::

                >>> show(clarinet.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of clarinet in B-flat's written middle C.

        ..  container:: example

            ::

                >>> clarinet.sounding_pitch_of_written_middle_c
                NamedPitch('bf')

            ::

                >>> show(clarinet.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        '''
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)



from abjad import *


def test_spannertools_TextSpanner__make_storage_format_with_overrides_01():

    text_spanner_1 = spannertools.TextSpanner()
    override(text_spanner_1).text_spanner.color = 'red'
    override(text_spanner_1).text_spanner.dash_fraction = 0.5

    staff_1 = Staff("c'8 d'8 e'8 f'8")
    attach(text_spanner_1, staff_1[:])

    assert systemtools.TestManager.compare(
        staff_1,
        r'''
        \new Staff {
            \override TextSpanner #'color = #red
            \override TextSpanner #'dash-fraction = #0.5
            c'8 \startTextSpan
            d'8
            e'8
            f'8 \stopTextSpan
            \revert TextSpanner #'color
            \revert TextSpanner #'dash-fraction
        }
        '''
        )

    string = text_spanner_1._make_storage_format_with_overrides()
    text_spanner_2 = eval(string)

    staff_2 = Staff("c'8 d'8 e'8 f'8")
    attach(text_spanner_2, staff_2[:])

    assert override(text_spanner_1) == override(text_spanner_2)
    assert format(staff_1) == format(staff_2)


# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.Matrix import Matrix


# TODO: change CyclicMatrix(columns=[...]) to CyclicMatrix.from_columns(...)
class CyclicMatrix(Matrix):
    '''A cyclic matrix.

    Initializes from rows:

    ::

        >>> cyclic_matrix = datastructuretools.CyclicMatrix([
        ...     [0, 1, 2, 3],
        ...     [10, 11, 12, 13],
        ...     [20, 21, 22, 23],
        ...     ])

    ::

        >>> cyclic_matrix
        CyclicMatrix(3x4)

    ::

        >>> cyclic_matrix[2]
        CyclicTuple([20, 21, 22, 23])

    ::

        >>> cyclic_matrix[2][2]
        22

    ::

        >>> cyclic_matrix[99]
        CyclicTuple([0, 1, 2, 3])

    ::

        >>> cyclic_matrix[99][99]
        3

    Initializes from columns:

    ::

        >>> cyclic_matrix = datastructuretools.CyclicMatrix(columns=[
        ...     [0, 10, 20],
        ...     [1, 11, 21],
        ...     [2, 12, 22],
        ...     [3, 13, 23],
        ...     ])

    ::

        >>> cyclic_matrix
        CyclicMatrix(3x4)

    ::

        >>> cyclic_matrix[2]
        CyclicTuple([20, 21, 22, 23])

    ::

        >>> cyclic_matrix[2][2]
        22

    ::

        >>> cyclic_matrix[99]
        CyclicTuple([0, 1, 2, 3])

    ::

        >>> cyclic_matrix[99][99]
        3

    Only item retrieval is currently implemented.

    Concatenation and division remain to be implemented.

    Standard transforms of linear algebra remain to be implemented.
    '''

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        if len(args) == 1:
            assert not kwargs
            rows, columns = self._initialize_from_rows(args[0])
        elif 'columns' in kwargs:
            assert not args
            rows, columns = self._initialize_from_columns(kwargs['columns'])
        elif len(args) == 0:
            rows, columns = self._initialize_from_rows([[0],[0]])
        else:
            message = 'can not initialize {}: {!r}.'
            message = message(type(self).__name__, args)
            raise ValueError(message)
        self._rows = rows
        self._columns = columns
        self._n_rows = len(rows)
        self._n_columns = len(columns)

    ### SPECIAL METHODS ###

    def __getitem__(self, i):
        r'''Gets row `i` from cyclic matrix.

        Returns row.
        '''
        return self.rows[i]

    def __repr__(self):
        r'''Gets interpreter representation of cyclic matrix.

        Returns string.
        '''
        return '{}({}x{})'.format(
            type(self).__name__,
            self._n_rows,
            self._n_columns,
            )

    ### PRIVATE METHODS ###

    def _initialize_from_columns(self, columns):
        from abjad.tools import datastructuretools
        columns = datastructuretools.CyclicTuple([
            datastructuretools.CyclicTuple(column)
            for column in columns
            ])
        rows = []
        for row_index in range(len(columns[0])):
            row = datastructuretools.CyclicTuple([
                column[row_index] for column in columns
                ])
            rows.append(row)
        rows = datastructuretools.CyclicTuple(rows)
        return rows, columns

    def _initialize_from_rows(self, rows):
        from abjad.tools import datastructuretools
        rows = datastructuretools.CyclicTuple([
            datastructuretools.CyclicTuple(row) for row in rows
            ])
        columns = []
        for column_index in range(len(rows[0])):
            column = datastructuretools.CyclicTuple([
                row[column_index] for row in rows
                ])
            columns.append(column)
        columns = datastructuretools.CyclicTuple(columns)
        return rows, columns

    ### PUBLIC PROPERTIES ###

    @property
    def columns(self):
        r'''Columns of cyclic matrix.

        ::

            >>> cyclic_matrix = datastructuretools.CyclicMatrix([
            ...     [0, 1, 2, 3],
            ...     [10, 11, 12, 13],
            ...     [20, 21, 22, 23],
            ...     ])

        ::

            >>> print(format(cyclic_matrix.columns))
            datastructuretools.CyclicTuple(
                [
                    datastructuretools.CyclicTuple(
                        [0, 10, 20]
                        ),
                    datastructuretools.CyclicTuple(
                        [1, 11, 21]
                        ),
                    datastructuretools.CyclicTuple(
                        [2, 12, 22]
                        ),
                    datastructuretools.CyclicTuple(
                        [3, 13, 23]
                        ),
                    ]
                )

        Returns cyclic tuple.
        '''
        return self._columns

    @property
    def rows(self):
        r'''Rows of cyclic matrix.

        ::

            >>> cyclic_matrix = datastructuretools.CyclicMatrix([
            ...     [0, 1, 2, 3],
            ...     [10, 11, 12, 13],
            ...     [20, 21, 22, 23],
            ...     ])

        ::

            >>> print(format(cyclic_matrix.rows))
            datastructuretools.CyclicTuple(
                [
                    datastructuretools.CyclicTuple(
                        [0, 1, 2, 3]
                        ),
                    datastructuretools.CyclicTuple(
                        [10, 11, 12, 13]
                        ),
                    datastructuretools.CyclicTuple(
                        [20, 21, 22, 23]
                        ),
                    ]
                )

        Returns cyclic tuple.
        '''
        return self._rows



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def truncate_sequence(sequence, sum_=None, weight=None):
    '''Truncates `sequence`.

    ..  container:: example

        Example sequence:

        ::

            >>> sequence = [-1, 2, -3, 4, -5, 6, -7, 8, -9, 10]

    ..  container:: example

        Truncates sequence to weights ranging from 1 to 10:

        ::

            >>> for n in range(1, 11):
            ...     result = sequencetools.truncate_sequence(sequence, weight=n)
            ...     print(n, result)
            ... 
            1 [-1]
            2 [-1, 1]
            3 [-1, 2]
            4 [-1, 2, -1]
            5 [-1, 2, -2]
            6 [-1, 2, -3]
            7 [-1, 2, -3, 1]
            8 [-1, 2, -3, 2]
            9 [-1, 2, -3, 3]
            10 [-1, 2, -3, 4]

    ..  container:: example

        Truncates sequence to sums ranging from 1 to 10:

        ::

            >>> for n in range(1, 11):
            ...     result = sequencetools.truncate_sequence(sequence, sum_=n)
            ...     print(n, result)
            ... 
            1 [-1, 2]
            2 [-1, 2, -3, 4]
            3 [-1, 2, -3, 4, -5, 6]
            4 [-1, 2, -3, 4, -5, 6, -7, 8]
            5 [-1, 2, -3, 4, -5, 6, -7, 8, -9, 10]
            6 [-1, 2, -3, 4, -5, 6, -7, 8, -9, 10]
            7 [-1, 2, -3, 4, -5, 6, -7, 8, -9, 10]
            8 [-1, 2, -3, 4, -5, 6, -7, 8, -9, 10]
            9 [-1, 2, -3, 4, -5, 6, -7, 8, -9, 10]
            10 [-1, 2, -3, 4, -5, 6, -7, 8, -9, 10]

    ..  container:: example

        Truncates sequence to zero weight:

        ::

            >>> sequencetools.truncate_sequence(sequence, weight=0)
            []

    ..  container:: example

        Truncates sequence to zero sum:

        ::

            >>> sequencetools.truncate_sequence(sequence, sum_=0)
            []

    Ignores `sum_` when `weight` and `sum_` are both set.

    Raises type error when `sequence` is not a list.

    Raises value error on negative `sum_`.

    Returns new object of `sequence` type.
    '''

    if not isinstance(sequence, list):
        raise TypeError

    if weight is not None:
        if weight < 0:
            raise ValueError
        result = []
        if 0 < weight:
            total = 0
            for element in sequence:
                total += abs(element)
                if total < weight:
                    result.append(element)
                else:
                    sign = mathtools.sign(element)
                    trimmed_part = weight - mathtools.weight(result)
                    trimmed_part *= sign
                    result.append(trimmed_part)
                    break
    elif sum_ is not None:
        sum_ = sum_
        if sum_ < 0:
            raise ValueError
        result = []
        if 0 < sum_:
            total = 0
            for element in sequence:
                total += element
                if total < sum_:
                    result.append(element)
                else:
                    result.append(sum_ - sum(result))
                    break

    result = type(sequence)(result)
    return result



# -*- encoding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import new


class TupletRhythmMaker(RhythmMaker):
    r'''Tuplet rhythm-maker.

    ..  container:: example

        Makes tuplets with ``3:2`` leaf ratios:

        ::

            >>> maker = rhythmmakertools.TupletRhythmMaker(
            ...     tuplet_ratios=[(3, 2)],
            ...     )

        ::

            >>> divisions = [(1, 2), (3, 8), (5, 16)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> staff = maker._get_rhythmic_staff(lilypond_file)
            >>> f(staff)
            \new RhythmicStaff {
                {
                    \time 1/2
                    \times 4/5 {
                        c'4.
                        c'4
                    }
                }
                {
                    \time 3/8
                    \tweak #'text #tuplet-number::calc-fraction-text
                    \times 3/5 {
                        c'4.
                        c'4
                    }
                }
                {
                    \time 5/16
                    {
                        c'8. [
                        c'8 ]
                    }
                }
            }

    ..  container:: example

        Makes tuplets with alternating ``1:-1`` and ``3:1`` leaf ratios:

        ::

            >>> maker = rhythmmakertools.TupletRhythmMaker(
            ...     tuplet_ratios=[(1, -1), (3, 1)],
            ...     )

        ::

            >>> divisions = [(1, 2), (3, 8), (5, 16)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> staff = maker._get_rhythmic_staff(lilypond_file)
            >>> f(staff)
            \new RhythmicStaff {
                {
                    \time 1/2
                    {
                        c'4
                        r4
                    }
                }
                {
                    \time 3/8
                    \tweak #'text #tuplet-number::calc-fraction-text
                    \times 3/4 {
                        c'4.
                        c'8
                    }
                }
                {
                    \time 5/16
                    \tweak #'text #tuplet-number::calc-fraction-text
                    \times 5/6 {
                        c'8.
                        r8.
                    }
                }
            }

    Usage follows the two-step configure-once / call-repeatedly pattern shown
    here.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_tuplet_ratios',
        '_tuplet_spelling_specifier',
        )

    _class_name_abbreviation = 'T'

    _human_readable_class_name = 'tuplet rhythm-maker'

    ### INITIALIZER ###

    def __init__(
        self,
        tuplet_ratios=((1, 1), (1, 2), (1, 3)),
        beam_specifier=None,
        duration_spelling_specifier=None,
        tie_specifier=None,
        tuplet_spelling_specifier=None,
        ):
        RhythmMaker.__init__(
            self,
            beam_specifier=beam_specifier,
            duration_spelling_specifier=duration_spelling_specifier,
            tie_specifier=tie_specifier,
            )
        from abjad.tools import rhythmmakertools
        prototype = (rhythmmakertools.TupletSpellingSpecifier, type(None))
        assert isinstance(tuplet_spelling_specifier, prototype)
        if tuplet_ratios is not None:
            tuplet_ratios = tuple(mathtools.Ratio(x) for x in tuplet_ratios)
        self._tuplet_ratios = tuplet_ratios
        self._tuplet_spelling_specifier = tuplet_spelling_specifier

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls tuplet rhythm-maker on `divisions`.

        ..  container:: example

            ::

                >>> divisions = [(1, 2), (3, 8), (5, 16)]
                >>> music = maker(divisions)
                >>> for division in music:
                ...     division
                Selection(FixedDurationTuplet(Duration(1, 2), "c'4 r4"),)
                Selection(FixedDurationTuplet(Duration(3, 8), "c'4. c'8"),)
                Selection(FixedDurationTuplet(Duration(5, 16), "c'8. r8."),)

        Returns list of selections. Each selection holds a single
        fixed-duration tuplet.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            seeds=seeds,
            )

    def __format__(self, format_specification=''):
        r'''Formats tuplet rhythm-maker.

        ..  container:: example

            ::

                rhythmmakertools.TupletRhythmMaker(
                    tuplet_ratios=(
                        mathtools.Ratio(1, -1),
                        mathtools.Ratio(3, 1),
                        ),
                    )

        Set `format_specification` to `''` or `'storage'`.

        Returns string.
        '''
        superclass = super(TupletRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE METHODS ###

    def _make_music(self, duration_pairs, seeds):
        from abjad.tools import rhythmmakertools
        tuplets = []
        if not isinstance(seeds, int):
            seeds = 0
        tuplet_ratios = datastructuretools.CyclicTuple(
            sequencetools.rotate_sequence(self.tuplet_ratios, seeds)
            )
        beam_specifier = self.beam_specifier
        if beam_specifier is None:
            beam_specifier = rhythmmakertools.BeamSpecifier()
        tuplet_spelling_specifier = self.tuplet_spelling_specifier
        if tuplet_spelling_specifier is None:
            tuplet_spelling_specifier = \
                rhythmmakertools.TupletSpellingSpecifier()
        for duration_index, duration_pair in enumerate(duration_pairs):
            ratio = tuplet_ratios[duration_index]
            duration = durationtools.Duration(duration_pair)
            tuplet = self._make_tuplet(
                duration,
                ratio,
                avoid_dots=tuplet_spelling_specifier.avoid_dots,
                is_diminution=tuplet_spelling_specifier.is_diminution,
                )
            if beam_specifier.beam_each_division:
                beam = spannertools.MultipartBeam()
                attach(beam, tuplet)
            tuplets.append(tuplet)
        if beam_specifier.beam_divisions_together:
            beam = spannertools.MultipartBeam()
            attach(beam, tuplets)
        selections = [selectiontools.Selection(x) for x in tuplets]
        return selections

    def _make_tuplet(
        self,
        duration,
        ratio,
        avoid_dots=False,
        is_diminution=True,
        ):
        tuplet = scoretools.Tuplet.from_duration_and_ratio(
            duration,
            ratio,
            avoid_dots=avoid_dots,
            is_diminution=is_diminution,
            )
        return tuplet

    ### PUBLIC PROPERTIES ###

    @property
    def tie_specifier(self):
        r'''Gets tie specifier of ratio talea rhythm-maker.

        ..  container:: example

            ::

                >>> tie_specifier = rhythmmakertools.TieSpecifier(
                ...     tie_across_divisions=True,
                ...     )
                >>> maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(2, 3), (1, -2, 1)],
                ...     tie_specifier=tie_specifier,
                ...     )

            ::

                >>> divisions = [(1, 2), (3, 8), (5, 16)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 1/2
                        \times 4/5 {
                            c'4
                            c'4. ~
                        }
                    }
                    {
                        \time 3/8
                        {
                            c'16.
                            r8.
                            c'16. ~
                        }
                    }
                    {
                        \time 5/16
                        {
                            c'8 [
                            c'8. ]
                        }
                    }
                }

        Returns tie specifier.
        '''
        return RhythmMaker.tie_specifier.fget(self)

    @property
    def tuplet_ratios(self):
        r'''Gets ratio talea of tuplet rhythm-maker.

        ..  container:: example

            ::

                >>> maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(2, 3), (1, -2, 1)],
                ...     tie_specifier=tie_specifier,
                ...     )
                >>> maker.tuplet_ratios
                (Ratio(2, 3), Ratio(1, -2, 1))

        Returns tuple of ratios.
        '''
        return self._tuplet_ratios

    @property
    def tuplet_spelling_specifier(self):
        r'''Gets tuplet spelling specifier of tuplet rhythm-maker.

        Returns tuplet spelling specifier.
        '''
        return self._tuplet_spelling_specifier

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses tuplet rhythm-maker.

        ..  container:: example

            ::

                >>> maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(2, 3), (1, -2, 1)],
                ...     )
                >>> reversed_maker = maker.reverse()

            ::

                >>> print(format(reversed_maker))
                rhythmmakertools.TupletRhythmMaker(
                    tuplet_ratios=(
                        mathtools.Ratio(1, -2, 1),
                        mathtools.Ratio(3, 2),
                        ),
                    duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
                        decrease_durations_monotonically=False,
                        ),
                    )

            ::

                >>> divisions = [(1, 2), (3, 8), (5, 16)]
                >>> music = reversed_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 1/2
                        {
                            c'8
                            r4
                            c'8
                        }
                    }
                    {
                        \time 3/8
                        \tweak #'text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'4.
                            c'4
                        }
                    }
                    {
                        \time 5/16
                        \tweak #'text #tuplet-number::calc-fraction-text
                        \times 5/6 {
                            c'16.
                            r8.
                            c'16.
                        }
                    }
                }

        Defined equal to copy of maker with `tuplet_ratios` and
        `duration_spelling_specifier` reversed.

        Returns new tuplet rhythm-maker.
        '''
        from abjad.tools import rhythmmakertools
        tuplet_ratios = []
        for ratio in reversed(self.tuplet_ratios):
            ratio = mathtools.Ratio([x for x in reversed(ratio)])
            tuplet_ratios.append(ratio)
        tuplet_ratios = tuple(tuplet_ratios)
        specifier = self.duration_spelling_specifier
        if specifier is None:
            specifier = rhythmmakertools.DurationSpellingSpecifier()
        specifier = specifier.reverse()
        maker = new(
            self,
            tuplet_ratios=tuplet_ratios,
            duration_spelling_specifier=specifier,
            )
        return maker

    def rotate(self, n=0):
        r'''Rotates tuplet rhythm-maker.

        ..  container:: example

            ::

                >>> maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(2, 3), (1, -2, 1), (1, 2, 3)],
                ...     )
                >>> rotated_maker = maker.rotate(n=1)

            ::

                >>> print(format(rotated_maker))
                rhythmmakertools.TupletRhythmMaker(
                    tuplet_ratios=(
                        mathtools.Ratio(3, 1, 2),
                        mathtools.Ratio(3, 2),
                        mathtools.Ratio(1, 1, -2),
                        ),
                    )

            ::

                >>> divisions = [(1, 2), (3, 8), (5, 16)]
                >>> music = rotated_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 1/2
                        \times 2/3 {
                            c'4.
                            c'8
                            c'4
                        }
                    }
                    {
                        \time 3/8
                        \tweak #'text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'4.
                            c'4
                        }
                    }
                    {
                        \time 5/16
                        \tweak #'text #tuplet-number::calc-fraction-text
                        \times 5/6 {
                            c'16. [
                            c'16. ]
                            r8.
                        }
                    }
                }

        Returns new tuplet rhythm-maker.
        '''
        from abjad.tools import rhythmmakertools
        tuplet_ratios = []
        for ratio in sequencetools.rotate_sequence(self.tuplet_ratios, n):
            ratio = sequencetools.rotate_sequence(ratio, n)
            ratio = mathtools.Ratio(ratio)
            tuplet_ratios.append(ratio)
        tuplet_ratios = tuple(tuplet_ratios)
        maker = new(
            self,
            tuplet_ratios=tuplet_ratios,
            )
        return maker


# -*- encoding: utf-8 -*-


def parse(arg, language='english'):
    r'''Parses `arg` as LilyPond string.

    ::

        >>> parse("{c'4 d'4 e'4 f'4}")
        Container("c'4 d'4 e'4 f'4")

    ::

        >>> container = _

    ::

        >>> print(format(container))
        {
            c'4
            d'4
            e'4
            f'4
        }

    A pitch-name language may also be specified.

    ::

        >>> parse("{c'8 des' e' fis'}", language='nederlands')
        Container("c'8 df'8 e'8 fs'8")

    Returns Abjad expression.
    '''
    from abjad.tools import rhythmtreetools
    from abjad.tools import lilypondparsertools

    if arg.startswith('abj:'):
        return lilypondparsertools.parse_reduced_ly_syntax(arg[4:])
    elif arg.startswith('rtm:'):
        return rhythmtreetools.parse_rtm_syntax(arg[4:])
    return lilypondparsertools.LilyPondParser(default_language=language)(arg)



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_set_written_pitch_of_pitched_components_in_expr_01():

    staff = Staff("c' d' e' f'")

    r'''
    \new Staff {
        c'4
        d'4
        e'4
        f'4
    }
    '''

    pitchtools.set_written_pitch_of_pitched_components_in_expr(staff)

    r'''
    \new Staff {
        c'4
        c'4
        c'4
        c'4
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'4
            c'4
            c'4
            c'4
        }
        '''
        )


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_PitchArrayCell___init___01():
    r'''Initializeempty.
    '''

    cell = pitchtools.PitchArrayCell()
    assert cell.pitches == []
    assert cell.width == 1


def test_pitchtools_PitchArrayCell___init___02():
    r'''Initialize with positive integer width.
    '''

    cell = pitchtools.PitchArrayCell(2)
    assert cell.pitches == []
    assert cell.width == 2


def test_pitchtools_PitchArrayCell___init___03():
    r'''Initialize with pitch instance.
    '''

    cell = pitchtools.PitchArrayCell(NamedPitch(0))
    assert cell.pitches == [NamedPitch(0)]
    assert cell.width == 1


def test_pitchtools_PitchArrayCell___init___04():
    r'''Initialize with list of pitch items.
    '''

    cell = pitchtools.PitchArrayCell([0, 2, 4])
    assert cell.pitches == [NamedPitch(0), NamedPitch(2), NamedPitch(4)]
    assert cell.width == 1


def test_pitchtools_PitchArrayCell___init___05():
    r'''Initialize with list of pitch instances.
    '''

    cell = pitchtools.PitchArrayCell([NamedPitch(0), NamedPitch(2), NamedPitch(4)])
    assert cell.pitches == [NamedPitch(0), NamedPitch(2), NamedPitch(4)]
    assert cell.width == 1


def test_pitchtools_PitchArrayCell___init___06():
    r'''Initialize with list of pitch pairs.
    '''

    cell = pitchtools.PitchArrayCell([('c', 4), ('d', 4), ('e', 4)])
    assert cell.pitches == [NamedPitch(0), NamedPitch(2), NamedPitch(4)]
    assert cell.width == 1


def test_pitchtools_PitchArrayCell___init___07():
    r'''Initialize with pitch item, width pair.
    '''

    cell = pitchtools.PitchArrayCell((0, 2))
    assert cell.pitches == [NamedPitch(0)]
    assert cell.width == 2


def test_pitchtools_PitchArrayCell___init___08():
    r'''Initialize with pitch instance, width pair.
    '''

    cell = pitchtools.PitchArrayCell((NamedPitch(0), 2))
    assert cell.pitches == [NamedPitch(0)]
    assert cell.width == 2


def test_pitchtools_PitchArrayCell___init___09():
    r'''Initialize with pitch item list, width pair.
    '''

    cell = pitchtools.PitchArrayCell(([0, 2, 4], 2))
    assert cell.pitches == [NamedPitch(0), NamedPitch(2), NamedPitch(4)]
    assert cell.width == 2


def test_pitchtools_PitchArrayCell___init___10():
    r'''Initialize with pitch instance list, width pair.
    '''

    cell = pitchtools.PitchArrayCell(([NamedPitch(0), NamedPitch(2), NamedPitch(4)], 2))
    assert cell.pitches == [NamedPitch(0), NamedPitch(2), NamedPitch(4)]
    assert cell.width == 2


# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.pitchtools.Segment import Segment
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import new


class PitchClassSegment(Segment):
    '''A pitch-class segment.

    ::

        >>> numbered_pitch_class_segment = pitchtools.PitchClassSegment(
        ...     items=[-2, -1.5, 6, 7, -1.5, 7],
        ...     item_class=pitchtools.NumberedPitchClass,
        ...     )
        >>> numbered_pitch_class_segment
        PitchClassSegment([10, 10.5, 6, 7, 10.5, 7])

    ::

        >>> named_pitch_class_segment = pitchtools.PitchClassSegment(
        ...     items=['c', 'ef', 'bqs,', 'd'],
        ...     item_class=pitchtools.NamedPitchClass,
        ...     )
        >>> named_pitch_class_segment
        PitchClassSegment(['c', 'ef', 'bqs', 'd'])

    Pitch-class segments are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=(-2, -1.5, 6, 7, -1.5, 7),
        item_class=None,
        ):
        Segment.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.PitchClass

    ### PUBLIC METHODS ###

    def alpha(self):
        r'''Morris alpha transform of pitch-class segment:

        ::

            >>> pitch_class_segment = pitchtools.PitchClassSegment(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_segment.alpha()
            PitchClassSegment([11, 11.5, 7, 6, 11.5, 6])

        Returns new pitch-class segment.
        '''
        from abjad.tools import mathtools
        numbers = []
        for pc in self:
            pc = abs(float(pc))
            is_integer = True
            if not mathtools.is_integer_equivalent_number(pc):
                is_integer = False
                fraction_part = pc - int(pc)
                pc = int(pc)
            if abs(pc) % 2 == 0:
                number = (abs(pc) + 1) % 12
            else:
                number = abs(pc) - 1
            if not is_integer:
                number += fraction_part
            else:
                number = int(number)
            numbers.append(number)
        return new(self, items=numbers)

    @classmethod
    def from_selection(cls, selection, item_class=None):
        r'''Initialize pitch-class segment from component selection:

        ::

            >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = Staff("c4. r8 g2")
            >>> selection = select((staff_1, staff_2))
            >>> pitchtools.PitchClassSegment.from_selection(selection)
            PitchClassSegment(['c', 'd', 'fs', 'a', 'b', 'c', 'g'])

        Returns pitch-class segment.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return cls(
            items=pitch_segment,
            item_class=item_class,
            )

    def invert(self):
        r'''Invert pitch-class segment:

        ::

            >>> pitch_class_segment = pitchtools.PitchClassSegment(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_segment.invert()
            PitchClassSegment([2, 1.5, 6, 5, 1.5, 5])

        Returns new pitch-class segment.
        '''
        items = (pc.invert() for pc in self)
        return new(self, items=items)

    def is_equivalent_under_transposition(self, expr):
        r'''True if equivalent under transposition to `expr`. Otherwise False.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        if not isinstance(expr, type(self)):
            return False
        if not len(self) == len(expr):
            return False
        difference = -(pitchtools.NamedPitch(expr[0], 4) -
            pitchtools.NamedPitch(self[0], 4))
        new_pitch_classes = (x + difference for x in self)
        new_pitch_classes = new(self, items=new_pitch_classes)
        return expr == new_pitch_classes

    def make_notes(self, n=None, written_duration=None):
        r'''Make first `n` notes in pitch class segment.

        Set `n` equal to `n` or length of segment.

        Set `written_duration` equal to `written_duration` or ``1/8``:

        ::

            >>> pitch_class_segment = pitchtools.PitchClassSegment(
            ...     [2, 4.5, 6, 11, 4.5, 10])

        ::

            >>> notes = pitch_class_segment.make_notes()
            >>> staff = Staff(notes)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                d'8
                eqs'8
                fs'8
                b'8
                eqs'8
                bf'8
            }

        Allow nonassignable `written_duration`:

        ::

            >>> notes = pitch_class_segment.make_notes(4, Duration(5, 16))
            >>> staff = Staff(notes)
            >>> time_signature = TimeSignature((5, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \time 5/4
                d'4 ~
                d'16
                eqs'4 ~
                eqs'16
                fs'4 ~
                fs'16
                b'4 ~
                b'16
            }

        Returns list of notes.
        '''
        from abjad.tools import scoretools
        from abjad.tools import pitchtools
        n = n or len(self)
        written_duration = written_duration or durationtools.Duration(1, 8)
        result = scoretools.make_notes([0] * n, [written_duration])
        for i, logical_tie in enumerate(iterate(result).by_logical_tie()):
            pitch_class = pitchtools.NamedPitchClass(self[i % len(self)])
            pitch = pitchtools.NamedPitch(pitch_class, 4)
            for note in logical_tie:
                note.written_pitch = pitch
        return result

    def multiply(self, n):
        r'''Multiply pitch-class segment by `n`:

        ::

            >>> pitch_class_segment = pitchtools.PitchClassSegment(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_segment.multiply(5)
            PitchClassSegment([2, 4.5, 6, 11, 4.5, 11])

        Returns new pitch-class segment.
        '''
        from abjad.tools import pitchtools
        items = (pitchtools.NumberedPitchClass(pc).multiply(n)
            for pc in self)
        return new(self, items=items)

    def retrograde(self):
        r'''Retrograde of pitch-class segment:

        ::

            >>> pitchtools.PitchClassSegment(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     ).retrograde()
            PitchClassSegment([7, 10.5, 7, 6, 10.5, 10])

        Returns new pitch-class segment.
        '''
        return new(self, items=reversed(self))

    def rotate(self, n, transpose=False):
        r'''Rotate pitch-class segment:

        ::

            >>> pitchtools.PitchClassSegment(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     ).rotate(1)
            PitchClassSegment([7, 10, 10.5, 6, 7, 10.5])

        ::

            >>> pitchtools.PitchClassSegment(
            ...     items=['c', 'ef', 'bqs,', 'd'],
            ...     ).rotate(-2)
            PitchClassSegment(['bqs', 'd', 'c', 'ef'])

        If `transpose` is true, transpose the rotated segment to begin at the
        same pitch class as this segment:

        ::

            >>> pitchtools.PitchClassSegment(
            ...     items=['c', 'b', 'd']
            ...     ).rotate(1, transpose=True)
            PitchClassSegment(['c', 'bf', 'a'])

        Returns new pitch-class segment.
        '''
        from abjad.tools import sequencetools
        items = sequencetools.rotate_sequence(self._collection, n)
        new_segment = new(self, items=items)
        if transpose:
            interval_of_transposition = float(self[0]) - float(new_segment[0])
            new_segment = new_segment.transpose(interval_of_transposition)
        return new_segment

    def transpose(self, expr):
        r'''Transpose pitch-class segment:

        ::

            >>> pitchtools.PitchClassSegment(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     ).transpose(10)
            PitchClassSegment([8, 8.5, 4, 5, 8.5, 5])

        Returns new pitch-class segment.
        '''
        items = (pitch_class.transpose(expr) for pitch_class in self)
        return new(self, items=items)

    def voice_horizontally(self, initial_octave=4):
        r'''Voices pitch-class segment as pitch segment, with each pitch as
        close in distance to the previous pitch as possible.

        ::

            >>> pitch_classes = pitchtools.PitchClassSegment(
            ...     "c b d e f g e b a c")
            >>> pitch_segment = pitch_classes.voice_horizontally()
            >>> show(pitch_segment) # doctest: +SKIP

        Returns pitch segment.
        '''
        from abjad.tools import pitchtools
        initial_octave = pitchtools.Octave(initial_octave)
        pitches = []
        if self:
            pitch_class = pitchtools.NamedPitchClass(self[0])
            pitch = pitchtools.NamedPitch(pitch_class, initial_octave)
            pitches.append(pitch)
            for pitch_class in self[1:]:
                pitch_class = pitchtools.NamedPitchClass(pitch_class)
                pitch = pitchtools.NamedPitch(pitch_class, initial_octave)
                semitones = abs((pitch - pitches[-1]).semitones)
                while 6 < semitones:
                    if pitch < pitches[-1]:
                        pitch += 12
                    else:
                        pitch -= 12
                    semitones = abs((pitch - pitches[-1]).semitones)
                pitches.append(pitch)
        if self.item_class is pitchtools.NamedPitchClass:
            item_class = pitchtools.NamedPitch
        else:
            item_class = pitchtools.NumberedPitch
        return pitchtools.PitchSegment(
            items=pitches,
            item_class=item_class,
            )

    def voice_vertically(self, initial_octave=4):
        r'''Voices pitch-class segment as pitch segment, with each pitch always
        higher than the previous.

        ::

            >>> scale_degree_numbers = [1, 3, 5, 7, 9, 11, 13]
            >>> scale = tonalanalysistools.Scale('c', 'minor')
            >>> pitch_classes = pitchtools.PitchClassSegment((
            ...     scale.scale_degree_to_named_pitch_class(x)
            ...     for x in scale_degree_numbers))
            >>> pitch_segment = pitch_classes.voice_vertically()
            >>> pitch_segment
            PitchSegment(["c'", "ef'", "g'", "bf'", "d''", "f''", "af''"])
            >>> show(pitch_segment) # doctest: +SKIP

        Returns pitch segment.
        '''
        from abjad.tools import pitchtools
        initial_octave = pitchtools.Octave(initial_octave)
        pitches = []
        if self:
            pitch_class = pitchtools.NamedPitchClass(self[0])
            pitch = pitchtools.NamedPitch(pitch_class, initial_octave)
            pitches.append(pitch)
            for pitch_class in self[1:]:
                pitch_class = pitchtools.NamedPitchClass(pitch_class)
                pitch = pitchtools.NamedPitch(pitch_class, initial_octave)
                while pitch < pitches[-1]:
                    pitch += 12
                pitches.append(pitch)
        if self.item_class is pitchtools.NamedPitchClass:
            item_class = pitchtools.NamedPitch
        else:
            item_class = pitchtools.NumberedPitch
        return pitchtools.PitchSegment(
            items=pitches,
            item_class=item_class,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def has_duplicates(self):
        r'''True if segment contains duplicate items:

        ::

            >>> pitch_class_segment = pitchtools.PitchClassSegment(
            ...     items=[-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_segment.has_duplicates
            True

        ::

            >>> pitch_class_segment = pitchtools.PitchClassSegment(
            ...     items="c d e f g a b",
            ...     )
            >>> pitch_class_segment.has_duplicates
            False

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        return len(pitchtools.PitchClassSet(self)) < len(self)


# -*- encoding: utf-8 -*-
import copy
import types
from abjad.tools.topleveltools import iterate


class Selection(object):
    r'''A selection of components.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_music',
        )

    ### INITIALIZER ###

    def __init__(self, music=None):
        music = self._coerce_music(music)
        self._music = tuple(music)

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        r'''Cocatenates `expr` to selection.

        Returns new selection.
        '''
        assert isinstance(expr, (Selection, list, tuple))
        if isinstance(expr, Selection):
            music = self._music + expr._music
            return type(self)(music)
        elif isinstance(expr, (tuple, list)):
            music = self._music + tuple(expr)
        return type(self)(music)

    def __contains__(self, expr):
        r'''Is true when `expr` is in selection. Otherwise false.

        Returns boolean.
        '''
        return expr in self._music

    def __eq__(self, expr):
        r'''Is true when selection and `expr` are of the same type
        and when music of selection equals music of `expr`.
        Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self._music == expr._music
        elif isinstance(expr, (list, tuple)):
            return self._music == tuple(expr)

    def __format__(self, format_specification=''):
        r'''Formats duration.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getitem__(self, expr):
        r'''Gets item `expr` from selection.

        Returns component from selection.
        '''
        result = self._music.__getitem__(expr)
        if isinstance(result, tuple):
            selection = type(self)()
            selection._music = result[:]
            result = selection
        return result

    def __hash__(self):
        r'''Hashes selection.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Selection, self).__hash__()

    def __illustrate__(self):
        r'''Attempts to illustrate selection.

        Evaluates the storage format of the selection (to sever any references
        to the source score from which the selection was taken). Then tries to
        wrap the result in a staff; in the case that notes of only C4 are found
        then sets the staff context name to ``'RhythmicStaff'``. If this works
        then the staff is wrapped in a LilyPond file and the file is returned.
        If this doesn't work then the method raises an exception.

        The idea is that the illustration should work for simple selections of
        that represent an essentially contiguous snippet of a single voice of
        music.
    
        Returns LilyPond file.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import mutate
        music = mutate(self).copy()
        staff = scoretools.Staff(music)
        found_different_pitch = False
        for pitch in pitchtools.list_named_pitches_in_expr(staff):
            if pitch != pitchtools.NamedPitch("c'"):
                found_different_pitch = True
                break
        if not found_different_pitch:
            staff.context_name = 'RhythmicStaff'
        score = scoretools.Score([staff])
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        return lilypond_file

    def __len__(self):
        r'''Number of components in selection.

        Returns nonnegative integer.
        '''
        return len(self._music)

    def __ne__(self, expr):
        r'''Is true when selection does not equal `expr`. Otherwise false.

        Returns boolean.
        '''
        return not self == expr

    def __radd__(self, expr):
        r'''Concatenates selection to `expr`.

        Returns newly created selection.
        '''
        assert isinstance(expr, (type(self), list, tuple))
        if isinstance(expr, type(self)):
            music = expr._music + self._music
            return Selection(music)
        # eventually remove this permissive branch
        # and force the use of selections only
        elif isinstance(expr, (tuple, list)):
            music = tuple(expr) + self._music
        return Selection(music)

    def __repr__(self):
        r'''Gets interpreter representation of selection.

        Returns string.
        '''
        return '{}{!r}'.format(type(self).__name__, self._music)

    ### PRIVATE PROPERTIES ###

    @property
    def _preprolated_duration(self):
        return sum(component._preprolated_duration for component in self)

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        if self._music:
            positional_argument_values = (
                self._music,
                )
        else:
            positional_argument_values = ()
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PRIVATE METHODS ###

    @staticmethod
    def _all_are_components_in_same_logical_voice(
        expr, prototype=None, allow_orphans=True):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        allowable_types = (
            list,
            tuple,
            types.GeneratorType,
            selectiontools.Selection,
            )
        if not isinstance(expr, allowable_types):
            return False
        prototype = prototype or (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        if len(expr) == 0:
            return True
        all_are_orphans_of_correct_type = True
        if allow_orphans:
            for component in expr:
                if not isinstance(component, prototype):
                    all_are_orphans_of_correct_type = False
                    break
                if not component._get_parentage().is_orphan:
                    all_are_orphans_of_correct_type = False
                    break
            if all_are_orphans_of_correct_type:
                return True
        first = expr[0]
        if not isinstance(first, prototype):
            return False
        orphan_components = True
        if not first._get_parentage().is_orphan:
            orphan_components = False
        same_logical_voice = True
        first_signature = first._get_parentage().logical_voice
        for component in expr[1:]:
            parentage = component._get_parentage()
            if not parentage.is_orphan:
                orphan_components = False
            if not allow_orphans and orphan_components:
                return False
            if parentage.logical_voice != first_signature:
                same_logical_voice = False
            if not allow_orphans and not same_logical_voice:
                return False
            if allow_orphans and not orphan_components and \
                not same_logical_voice:
                return False
        return True

    @staticmethod
    def _all_are_contiguous_components_in_same_logical_voice(
        expr, prototype=None, allow_orphans=True):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        allowable_types = (
            list,
            tuple,
            types.GeneratorType,
            selectiontools.Selection,
            )
        if not isinstance(expr, allowable_types):
            return False
        prototype = prototype or (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        if len(expr) == 0:
            return True
        all_are_orphans_of_correct_type = True
        if allow_orphans:
            for component in expr:
                if not isinstance(component, prototype):
                    all_are_orphans_of_correct_type = False
                    break
                if not component._get_parentage().is_orphan:
                    all_are_orphans_of_correct_type = False
                    break
            if all_are_orphans_of_correct_type:
                return True
        if not allow_orphans:
            if any(x._get_parentage().is_orphan for x in expr):
                return False
        first = expr[0]
        if not isinstance(first, prototype):
            return False
        first_parentage = first._get_parentage()
        first_logical_voice = first_parentage.logical_voice
        first_root = first_parentage.root
        previous = first
        for current in expr[1:]:
            current_parentage = current._get_parentage()
            current_logical_voice = \
                current_parentage.logical_voice
            # false if wrong type of component found
            if not isinstance(current, prototype):
                return False
            # false if in different logical voices
            if current_logical_voice != first_logical_voice:
                return False
            # false if components are in same score and are discontiguous
            if current_parentage.root == first_root:
                if not previous._is_immediate_temporal_successor_of(current):
                    return False
            previous = current
        return True

    @staticmethod
    def _all_are_contiguous_components_in_same_parent(
        expr, prototype=None, allow_orphans=True):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        allowable_types = (
            list,
            tuple,
            types.GeneratorType,
            selectiontools.Selection,
            )
        if not isinstance(expr, allowable_types):
            return False
        prototype = prototype or (scoretools.Component, )
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        if len(expr) == 0:
            return True
        all_are_orphans_of_correct_type = True
        if allow_orphans:
            for component in expr:
                if not isinstance(component, prototype):
                    all_are_orphans_of_correct_type = False
                    break
                if not component._get_parentage().is_orphan:
                    all_are_orphans_of_correct_type = False
                    break
            if all_are_orphans_of_correct_type:
                return True
        first = expr[0]
        if not isinstance(first, prototype):
            return False
        first_parent = first._parent
        if first_parent is None:
            if allow_orphans:
                orphan_components = True
            else:
                return False
        same_parent = True
        strictly_contiguous = True
        previous = first
        for current in expr[1:]:
            if not isinstance(current, prototype):
                return False
            if not current._get_parentage().is_orphan:
                orphan_components = False
            if not current._parent is first_parent:
                same_parent = False
            if not previous._is_immediate_temporal_successor_of(current):
                strictly_contiguous = False
            if (not allow_orphans or
                (allow_orphans and not orphan_components)) and \
                (not same_parent or not strictly_contiguous):
                return False
            previous = current
        return True

    @staticmethod
    def _coerce_music(music):
        if music is None:
            music = ()
        elif isinstance(music, (tuple, list)):
            music = tuple(music)
        elif isinstance(music, Selection):
            music = tuple(music)
        elif isinstance(music, types.GeneratorType):
            music = tuple(music)
        else:
            music = (music, )
        return music

    def _get_component(self, prototype=None, n=0, recurse=True):
        from abjad.tools import scoretools
        prototype = prototype or (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        if 0 <= n:
            if recurse:
                components = iterate(self).by_class(prototype)
            else:
                components = self._music
            for i, x in enumerate(components):
                if i == n:
                    return x
        else:
            if recurse:
                components = iterate(self).by_class(
                    prototype, reverse=True)
            else:
                components = reversed(self._music)
            for i, x in enumerate(components):
                if i == abs(n) - 1:
                    return x

    def _get_spanner(self, prototype=None):
        spanners = self._get_spanners(prototype=prototype)
        if not spanners:
            message = 'no spanners found.'
            raise MissingSpannerError(message)
        elif len(spanners) == 1:
            return spanners.pop()
        else:
            message = 'multiple spanners found.'
            raise ExtraSpannerError(message)

    def _get_spanners(self, prototype=None):
        from abjad.tools import spannertools
        prototype = prototype or (spannertools.Spanner,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        result = set()
        for component in self:
            spanners = component._get_spanners(prototype)
            result.update(spanners)
        return result

    def _iterate_components(self, recurse=True, reverse=False):
        if recurse:
            return iterate(self).by_class()
        else:
            return self._iterate_top_level_components(reverse=reverse)

    def _iterate_top_level_components(self, reverse=False):
        if reverse:
            for component in reversed(self):
                yield component
        else:
            for component in self:
                yield component

    def _set_parents(self, new_parent):
        r'''Not composer-safe.
        '''
        for component in self._music:
            component._set_parent(new_parent)

    ### PUBLIC METHODS ###

    def get_duration(self, in_seconds=False):
        r'''Gets duration of contiguous selection.

        Returns duration.
        '''
        return sum(
            component._get_duration(in_seconds=in_seconds)
            for component in self
            )

    def get_spanners(self, prototype=None):
        r'''Gets spanners attached to any component in selection.

        Returns set.
        '''
        result = set()
        for component in self:
            spanners = component._get_spanners(prototype=prototype)
            result.update(spanners)
        return result


# -*- encoding: utf-8 -*-
r'''Source: PythonDecorateLibrary:
See: https://wiki.python.org/moin/PythonDecoratorLibrary#Memoize
'''
#import collections
#import functools
#from abjad.tools.abctools.AbjadObject import AbjadObject
#
#
#class Memoize(AbjadObject):
#    r'''Memoize decorator.
#
#    Caches a function's return value.
#    '''
#
#    ### INITIALIZER ###
#
#    def __init__(self, function=None):
#       self.function = function
#       self.cache = {}
#
#    ### SPECIAL METHODS ###
#
#    def __call__(self, *args):
#        r'''Calls decorator on `args`.
#
#        Returns cached value.
#        '''
#        if not isinstance(args, collections.Hashable):
#            # mutable objects can not cache;
#            # reevaluates function instead of blowing up
#            return self.function(*args)
#        if args in self.cache:
#            return self.cache[args]
#        else:
#            value = self.function(*args)
#            self.cache[args] = value
#            return value
#
#    def __get__(self, obj, objtype):
#        r'''Supports instance methods.
#        '''
#        return functools.partial(self.__call__, obj)


class Memoize(dict):
    r'''Memoize decorator.

    Caches function return value.
    '''

    ### INITIALIZER ###

    def __init__(self, function=None):
        self.function = function

    ### SPECIAL METHODS ###

    def __call__(self, *args):
        r'''Calls decorator on `args`.

        Calls function on `args` and caches if no cached value is found.

        Returns cached value.
        '''
        return self[args]

    def __missing__(self, key):
        r'''Calls function on `*key` and caches.

        Returns cached value.
        '''
        result = self[key] = self.function(*key)
        return result



# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


# TODO: replace kind='prolated' keyword with in_seconds=False keyword
def set_line_breaks_by_line_duration(
    expr,
    line_duration,
    line_break_class=None,
    kind='prolated',
    add_empty_bars=False,
    ):
    r'''Iterate `line_break_class` instances in `expr`
    and accumulate `kind` duration.

    Add line break after every total less than or equal to `line_duration`.

    Set `line_break_class` to measure when `line_break_class` is none.
    '''

    if line_break_class is None:
        line_break_class = scoretools.Measure

    previous = None
    cumulative_duration = durationtools.Duration(0)
    for current in iterate(expr).by_class(line_break_class):
        # TODO: compress these 4 lines to only the 4th line
        #       after duration migration
        if kind == 'seconds':
            current_duration = current._get_duration(in_seconds=True)
        elif kind == 'prolated':
            current_duration = current._get_duration()
        elif kind == 'preprolated':
            current_duration = current._preprolated_duration
        else:
            current_duration = getattr(current._get_duration(), kind)
        candidate_duration = cumulative_duration + current_duration
        if candidate_duration < line_duration:
            cumulative_duration += current_duration
        elif candidate_duration == line_duration:
            command = indicatortools.LilyPondCommand('break', 'closing')
            attach(command, current)
            if add_empty_bars:
                if current.bar_line.kind is None:
                    current.bar_line.kind = ''
            cumulative_duration = durationtools.Duration(0)
        else:
            if previous is not None:
                command = indicatortools.LilyPondCommand('break', 'closing')
                attach(command, previous)
                if add_empty_bars:
                    if current.bar_line.kind is None:
                        current.bar_line.kind = ''
            cumulative_duration = current_duration
        previous = current


lilypond_version = "2.19.6"

context_properties = [
    "aDueText",
    "accidentalGrouping",
    "additionalPitchPrefix",
    "alignAboveContext",
    "alignBassFigureAccidentals",
    "alignBelowContext",
    "alternativeNumberingStyle",
    "alternativeRestores",
    "associatedVoice",
    "autoAccidentals",
    "autoBeamCheck",
    "autoBeaming",
    "autoCautionaries",
    "automaticBars",
    "barAlways",
    "barCheckSynchronize",
    "barNumberFormatter",
    "barNumberVisibility",
    "baseMoment",
    "bassFigureFormatFunction",
    "bassStaffProperties",
    "beamExceptions",
    "beamHalfMeasure",
    "beatStructure",
    "chordChanges",
    "chordNameExceptions",
    "chordNameExceptionsFull",
    "chordNameExceptionsPartial",
    "chordNameFunction",
    "chordNameLowercaseMinor",
    "chordNameSeparator",
    "chordNoteNamer",
    "chordPrefixSpacer",
    "chordRootNamer",
    "clefGlyph",
    "clefPosition",
    "clefTransposition",
    "clefTranspositionFormatter",
    "clefTranspositionStyle",
    "completionBusy",
    "completionFactor",
    "completionUnit",
    "connectArpeggios",
    "countPercentRepeats",
    "createKeyOnClefChange",
    "createSpacing",
    "crescendoSpanner",
    "crescendoText",
    "cueClefGlyph",
    "cueClefPosition",
    "cueClefTransposition",
    "cueClefTranspositionFormatter",
    "cueClefTranspositionStyle",
    "currentBarNumber",
    "decrescendoSpanner",
    "decrescendoText",
    "defaultBarType",
    "defaultStrings",
    "doubleRepeatSegnoType",
    "doubleRepeatType",
    "doubleSlurs",
    "drumPitchTable",
    "drumStyleTable",
    "endRepeatSegnoType",
    "endRepeatType",
    "explicitClefVisibility",
    "explicitCueClefVisibility",
    "explicitKeySignatureVisibility",
    "extendersOverRests",
    "extraNatural",
    "figuredBassAlterationDirection",
    "figuredBassCenterContinuations",
    "figuredBassFormatter",
    "figuredBassPlusDirection",
    "fingeringOrientations",
    "firstClef",
    "followVoice",
    "fontSize",
    "forbidBreak",
    "forceClef",
    "fretLabels",
    "glissandoMap",
    "gridInterval",
    "handleNegativeFrets",
    "harmonicAccidentals",
    "harmonicDots",
    "highStringOne",
    "ignoreBarChecks",
    "ignoreFiguredBassRest",
    "ignoreMelismata",
    "implicitBassFigures",
    "implicitTimeSignatureVisibility",
    "includeGraceNotes",
    "instrumentCueName",
    "instrumentEqualizer",
    "instrumentName",
    "instrumentTransposition",
    "internalBarNumber",
    "keepAliveInterfaces",
    "keyAlterationOrder",
    "keySignature",
    "lyricMelismaAlignment",
    "majorSevenSymbol",
    "markFormatter",
    "maximumFretStretch",
    "measureLength",
    "measurePosition",
    "melismaBusyProperties",
    "metronomeMarkFormatter",
    "middleCClefPosition",
    "middleCCuePosition",
    "middleCOffset",
    "middleCPosition",
    "midiBalance",
    "midiChannelMapping",
    "midiChorusLevel",
    "midiInstrument",
    "midiMaximumVolume",
    "midiMergeUnisons",
    "midiMinimumVolume",
    "midiPanPosition",
    "midiReverbLevel",
    "minimumFret",
    "minimumPageTurnLength",
    "minimumRepeatLengthForPageTurn",
    "minorChordModifier",
    "noChordSymbol",
    "noteToFretFunction",
    "ottavation",
    "output",
    "partCombineTextsOnNote",
    "pedalSostenutoStrings",
    "pedalSostenutoStyle",
    "pedalSustainStrings",
    "pedalSustainStyle",
    "pedalUnaCordaStrings",
    "pedalUnaCordaStyle",
    "predefinedDiagramTable",
    "printKeyCancellation",
    "printOctaveNames",
    "printPartCombineTexts",
    "proportionalNotationDuration",
    "rehearsalMark",
    "repeatCommands",
    "repeatCountVisibility",
    "restCompletionBusy",
    "restNumberThreshold",
    "restrainOpenStrings",
    "searchForVoice",
    "segnoType",
    "shapeNoteStyles",
    "shortInstrumentName",
    "shortVocalName",
    "skipBars",
    "skipTypesetting",
    "slashChordSeparator",
    "soloIIText",
    "soloText",
    "squashedPosition",
    "staffLineLayoutFunction",
    "stanza",
    "startRepeatSegnoType",
    "startRepeatType",
    "stemLeftBeamCount",
    "stemRightBeamCount",
    "strictBeatBeaming",
    "stringNumberOrientations",
    "stringOneTopmost",
    "stringTunings",
    "strokeFingerOrientations",
    "subdivideBeams",
    "suggestAccidentals",
    "systemStartDelimiter",
    "systemStartDelimiterHierarchy",
    "tabStaffLineLayoutFunction",
    "tablatureFormat",
    "tempoHideNote",
    "tempoWholesPerMinute",
    "tieWaitForNote",
    "timeSignatureFraction",
    "timeSignatureSettings",
    "timing",
    "tonic",
    "topLevelAlignment",
    "trebleStaffProperties",
    "tupletFullLength",
    "tupletFullLengthNote",
    "tupletSpannerDuration",
    "useBassFigureExtenders",
    "vocalName",
    "voltaSpannerDuration",
    "whichBar",
]



# -*- encoding: utf-8 -*-
import math


def spell_pitch_number(pitch_number, diatonic_pitch_class_name):
    '''Spell `pitch_number` according to `diatonic_pitch_class_name`:

    ::

        >>> pitchtools.spell_pitch_number(14, 'c')
        (Accidental('ss'), 5)

    Returns accidental / octave-number pair.
    '''
    from abjad.tools import pitchtools

    # check input
    if not isinstance(pitch_number, (int, float)):
        raise TypeError

    if not isinstance(diatonic_pitch_class_name, str):
        raise TypeError

    if not diatonic_pitch_class_name in ['c', 'd', 'e', 'f', 'g', 'a', 'b']:
        raise ValueError

    # find accidental semitones
    pc = \
        pitchtools.PitchClass._diatonic_pitch_class_name_to_pitch_class_number[
            diatonic_pitch_class_name]
    nearest_neighbor = pitchtools.transpose_pitch_class_number_to_pitch_number_neighbor(
        pitch_number, pc)
    semitones = pitch_number - nearest_neighbor

    # find accidental alphabetic string
    abbreviation = \
        pitchtools.Accidental._semitones_to_abbreviation[
            semitones]
    accidental = pitchtools.Accidental(abbreviation)

    # find octave
    octave_number = int(math.floor((pitch_number - semitones) / 12)) + 4

    # return accidental and octave
    return accidental, octave_number


# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class WellformednessManager(AbjadObject):
    r'''Wellformedness manager.
    '''

    ### INITIALIZER ###

    def __init__(self, expr=None, allow_empty_containers=True):
        self.expr = expr
        self.allow_empty_containers = allow_empty_containers

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls all wellformedness checks on `expr`.

        Returns something.
        '''
        if self.expr is None:
            return
        check_names = [x for x in dir(self) if x.startswith('check_')]
        triples = []
        for current_check_name in sorted(check_names):
            if self.allow_empty_containers:
                if current_check_name == 'check_empty_containers':
                    continue
            current_check = getattr(self, current_check_name)
            current_violators, current_total = current_check()
            triple = (current_violators, current_total, current_check_name)
            triples.append(triple)
        return triples

    ### PUBLIC METHODS ###

    def check_beamed_quarter_notes(self):
        r'''Checks to make sure there are no beamed quarter notes.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        total = 0
        smart_beams = (
            spannertools.DuratedComplexBeam,
            spannertools.MultipartBeam,
            )
        for leaf in iterate(self.expr).by_class(scoretools.Leaf):
            total += 1
            parentage = leaf._get_parentage(include_self=True)
            beams = parentage._get_spanners(spannertools.Beam)
            for beam in beams:
                if not isinstance(beam, smart_beams):
                    flag_count = leaf.written_duration.flag_count
                    if flag_count < 1:
                        violators.append(leaf)
                        break
        return violators, total

    def check_discontiguous_spanners(self):
        r'''There are now two different types of spanner.
        Most spanners demand that spanner components be
        logical-voice-contiguous. But a few special spanners (like Tempo)
        do not make such a demand. The check here consults the experimental
        `_contiguity_constraint`.

        Returns violators and total.
        '''
        from abjad.tools.selectiontools import Selection
        violators = []
        total, bad = 0, 0
        for spanner in self.expr._get_descendants()._get_spanners():
            if spanner._contiguity_constraint == 'logical voice':
                if not Selection._all_are_contiguous_components_in_same_logical_voice(
                    spanner[:]):
                    violators.append(spanner)
            total += 1
        return violators, total

    def check_duplicate_ids(self):
        r'''Checks to make sure there are no components with duplicated IDs.

        Returns violators and total.
        '''
        from abjad.tools import sequencetools
        from abjad.tools.topleveltools import iterate
        violators = []
        components = iterate(self.expr).by_class()
        total_ids = [id(x) for x in components]
        unique_ids = sequencetools.remove_repeated_elements(total_ids)
        if len(unique_ids) < len(total_ids):
            for current_id in unique_ids:
                if 1 < total_ids.count(current_id):
                    violators.extend([x for x in components
                        if id(x) == current_id])
        return violators, len(total_ids)


    def check_empty_containers(self):
        r'''Checks to make sure there are no empty containers in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        violators = []
        bad, total = 0, 0
        for component in iterate(self.expr).by_class(scoretools.Container):
            if len(component) == 0:
                violators.append(component)
                bad += 1
            total += 1
        return violators, total

    def check_intermarked_hairpins(self):
        r'''Checks to make sure there are no hairpins in score with intervening
        dynamic marks.

        Returns violators and total.
        '''
        from abjad.tools import indicatortools
        from abjad.tools import spannertools
        violators = []
        total, bad = 0, 0
        prototype = (spannertools.Hairpin,)
        hairpins = self.expr._get_descendants()._get_spanners(prototype)
        for hairpin in hairpins:
            if 2 < len(hairpin._leaves):
                for leaf in hairpin._leaves[1:-1]:
                    if leaf._get_indicators(indicatortools.Dynamic):
                        violators.append(hairpin)
                        bad += 1
                        break
            total += 1
        return violators, total

    def check_misdurated_measures(self):
        r'''Checks to make sure there are no misdurated measures in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        violators = []
        total, bad = 0, 0
        for measure in iterate(self.expr).by_class(scoretools.Measure):
            time_signature = measure.time_signature
            if time_signature is not None:
                if measure._preprolated_duration != time_signature.duration:
                    violators.append(measure)
                    bad += 1
            total += 1
        return violators, total

    def check_misfilled_measures(self):
        r'''Checks that time signature duration equals measure contents
        duration for every measure.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        violators = []
        total, bad = 0, 0
        for measure in iterate(self.expr).by_class(scoretools.Measure):
            if measure.is_misfilled:
                violators.append(measure)
                bad += 1
            total += 1
        return violators, total

    def check_mispitched_ties(self):
        r'''Checks for mispitched notes. Does not check tied rests or skips.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        total = 0
        prototype = (spannertools.Tie,)
        for leaf in iterate(self.expr).by_class(scoretools.Note):
            total += 1
            spanners = leaf._get_spanners(prototype)
            if spanners:
                spanner = spanners.pop()
                if not spanner._is_my_last_leaf(leaf):
                    next_leaf = leaf._get_leaf(1)
                    if next_leaf:
                        if leaf.written_pitch != next_leaf.written_pitch:
                            violators.append(leaf)
        return violators, total


    def check_misrepresented_flags(self):
        r'''Checks to make sure there are no misrepresented flags in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        from abjad.tools.topleveltools import set_
        violators = []
        total = 0
        for leaf in iterate(self.expr).by_class(scoretools.Leaf):
            total += 1
            flags = leaf.written_duration.flag_count
            left = getattr(set_(leaf), 'stem_left_beam_count', None)
            right = getattr(set_(leaf), 'stem_right_beam_count', None)
            if left is not None:
                if flags < left or \
                    (left < flags and right not in (flags, None)):
                    if leaf not in violators:
                        violators.append(leaf)
            if right is not None:
                if flags < right or \
                    (right < flags and left not in (flags, None)):
                    if leaf not in violators:
                        violators.append(leaf)
        return violators, total

    def check_missing_parents(self):
        r'''Checks to make sure there are no components in score with missing
        parent.

        Returns violators and total.
        '''
        from abjad.tools.topleveltools import iterate
        violators = []
        total = 0
        components = iterate(self.expr).by_class()
        for i, component in enumerate(components):
            if 0 < i:
                if component._parent is None:
                    violators.append(component)
        total = i + 1
        return violators, total

    def check_nested_measures(self):
        r'''Checks to make sure there are no nested measures in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import iterate
        violators = []
        total = 0
        for measure in iterate(self.expr).by_class(scoretools.Measure):
            parentage = measure._get_parentage(include_self=False)
            if parentage.get_first(scoretools.Measure):
                violators.append(measure)
            total += 1
        return violators, total

    def check_overlapping_beams(self):
        r'''Checks to make sure there are no overlapping beams in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        prototype = (spannertools.Beam,)
        all_beams = set()
        for leaf in iterate(self.expr).by_class(scoretools.Leaf):
            beams = leaf._get_spanners(prototype)
            all_beams.update(beams)
            if 1 < len(beams):
                for beam in beams:
                    if beam not in violators:
                        violators.append(beam)
        total = len(all_beams)
        return violators, total

    def check_overlapping_glissandi(self):
        r'''Checks to make sure there are no overlapping glissandi in score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        prototype = (spannertools.Glissando,)
        for leaf in iterate(self.expr).by_class(scoretools.Leaf):
            glissandi = leaf._get_spanners(prototype)
            if 1 < len(glissandi):
                if len(glissandi) == 2:
                    common_leaves = set(glissandi[0]._leaves) & \
                        set(glissandi[1]._leaves)
                    if len(common_leaves) == 1:
                        x = list(common_leaves)[0]
                        if (glissandi[0]._is_my_first_leaf(x) and
                            glissandi[1]._is_my_last_leaf(x)) or \
                            (glissandi[1]._is_my_first_leaf(x) and
                            glissandi[0]._is_my_last_leaf(x)):
                            break

                for glissando in glissandi:
                    if glissando not in violators:
                        violators.append(glissando)
        total = self.expr._get_descendants()._get_spanners(prototype)
        total = len(total)
        return violators, total

    def check_overlapping_octavation_spanners(self):
        r'''Checks to make sure there are no overlapping octavation spanners in
        score.

        Returns violators and total.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import iterate
        violators = []
        prototype = (spannertools.OctavationSpanner, )
        for leaf in iterate(self.expr).by_class(scoretools.Leaf):
            spanners = leaf._get_descendants()._get_spanners(prototype)
            if 1 < len(spanners):
                for spanner in spanners:
                    if spanner not in violators:
                        violators.append(spanner)
        total = self.expr._get_descendants()._get_spanners(prototype)
        return violators, len(total)

    def check_short_hairpins(self):
        r'''Checks to make sure that hairpins span at least two leaves.

        Returns violators and total.
        '''
        from abjad.tools import spannertools
        violators = []
        total, bad = 0, 0
        prototype = (spannertools.Hairpin,)
        hairpins = self.expr._get_descendants()._get_spanners(prototype)
        for hairpin in hairpins:
            if len(hairpin._leaves) <= 1:
                violators.append(hairpin)
            total += 1
        return violators, total


# -*- encoding: utf-8 -*-
'''Dependencies:

The ``scoretools`` package should not import ``instrumenttools``
at top level.
'''
from abjad.tools import systemtools

systemtools.ImportManager.import_structured_package(
	__path__[0],
	globals(),
	)

_documentation_section = 'core'


# -*- encoding: utf-8 -*-
import sys


def is_string(expr):
    r'''Is true when `expr` is a string.

    Compatible under both Python 2.7.x and 3.x.
    '''
    if sys.version_info[0] == 2:
        if isinstance(expr, basestring):
            return True
    else:
        if isinstance(expr, str):
            return True
    return False



# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Instrument import Instrument


class Piano(Instrument):
    r'''A piano.

    ::

        >>> staff_group = StaffGroup()
        >>> staff_group.context_name = 'PianoStaff'
        >>> staff_group.append(Staff("c'4 d'4 e'4 f'4"))
        >>> staff_group.append(Staff("c'2 b2"))
        >>> piano = instrumenttools.Piano()
        >>> attach(piano, staff_group)
        >>> attach(Clef(name='bass'), staff_group[1])
        >>> show(staff_group) # doctest: +SKIP

    ..  doctest::

        >>> print(format(staff_group))
        \new PianoStaff <<
            \set PianoStaff.instrumentName = \markup { Piano }
            \set PianoStaff.shortInstrumentName = \markup { Pf. }
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }
            \new Staff {
                \clef "bass"
                c'2
                b2
            }
        >>

    The piano targets piano staff context by default.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='piano',
        short_instrument_name='pf.',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=('treble', 'bass'),
        pitch_range='[A0, C8]',
        sounding_pitch_of_written_middle_c=None,
        ):
        from abjad.tools import scoretools
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._default_scope = 'PianoStaff'
        self._performer_names.extend([
            'keyboardist',
            'pianist',
            ])
        self._is_primary_instrument = True

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets piano's allowable clefs.

        ..  container:: example

            ::

                >>> piano.allowable_clefs
                ClefInventory([Clef(name='treble'), Clef(name='bass')])

            ::

                >>> show(piano.allowable_clefs) # doctest: +SKIP

        Returns clef inventory.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def instrument_name(self):
        r'''Gets piano's name.

        ..  container:: example

            ::

                >>> piano.instrument_name
                'piano'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets piano's instrument name markup.

        ..  container:: example

            ::

                >>> piano.instrument_name_markup
                Markup(contents=('Piano',))

            ::

                >>> show(piano.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets piano's range.

        ..  container:: example

            ::

                >>> piano.pitch_range
                PitchRange(range_string='[A0, C8]')

            ::

                >>> show(piano.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets piano's short instrument name.

        ..  container:: example

            ::

                >>> piano.short_instrument_name
                'pf.'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets piano's short instrument name markup.

        ..  container:: example

            ::

                >>> piano.short_instrument_name_markup
                Markup(contents=('Pf.',))

            ::

                >>> show(piano.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of piano's written middle C.

        ..  container:: example

            ::

                >>> piano.sounding_pitch_of_written_middle_c
                NamedPitch("c'")

            ::

                >>> show(piano.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        '''
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)
